# -*- coding: utf-8 -*-

from pysped.xml_sped import *
from pysped.cte.leiaute import ESQUEMA_ATUAL_VERSAO_300 as ESQUEMA_ATUAL
import os

from .consrecicte_300 import ProtCTe as ProtCTe_300
from .canccte_300 import RetCancCTe as RetCancCTe_300
from .proceventocte_300 import ProcEventoCTe as ProcEventoCTe_300

DIRNAME = os.path.dirname(__file__)

class ConsSitCTe(XMLNFe):
    def __init__(self):
        super(ConsSitCTe, self).__init__()
        self.versao = TagDecimal(nome='consSitCTe', codigo='EP01', propriedade='versao', namespace=NAMESPACE_CTE, valor='3.00', raiz='/')
        self.tpAmb  = TagInteiro(nome='tpAmb'     , codigo='EP03', tamanho=[ 1,  1, 1], raiz='//consSitCTe', valor=2, namespace=NAMESPACE_CTE, namespace_obrigatorio=False)
        self.xServ  = TagCaracter(nome='xServ'    , codigo='EP04', tamanho=[ 9,  9]   , raiz='//consSitCTe', valor='CONSULTAR', namespace=NAMESPACE_CTE, namespace_obrigatorio=False)
        self.chCTe  = TagCaracter(nome='chCTe'    , codigo='EP05', tamanho=[44, 44]   , raiz='//consSitCTe', namespace=NAMESPACE_CTE, namespace_obrigatorio=False)
        self.caminho_esquema = os.path.join(DIRNAME, 'schema', ESQUEMA_ATUAL + '/')
        self.arquivo_esquema = 'consSitCTe_v3.00.xsd'

    def get_xml(self):
        xml = XMLNFe.get_xml(self)
        xml += ABERTURA
        xml += self.versao.xml
        xml += self.tpAmb.xml
        xml += self.xServ.xml
        xml += self.chCTe.xml
        xml += '</consSitCTe>'
        return xml

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.versao.xml = arquivo
            self.tpAmb.xml  = arquivo
            self.xServ.xml  = arquivo
            self.chCTe.xml  = arquivo

    xml = property(get_xml, set_xml)


class RetConsSitCTe(XMLNFe):
    def __init__(self):
        super(RetConsSitCTe, self).__init__()
        self.versao     = TagDecimal(nome='retConsSitCTe', codigo='ER01', propriedade='versao', namespace=NAMESPACE_CTE, valor='3.00', raiz='/')
        self.tpAmb      = TagInteiro(nome='tpAmb'        , codigo='ER03' , tamanho=[1,   1, 1], raiz='//retConsSitCTe', namespace=NAMESPACE_CTE, namespace_obrigatorio=False)
        self.verAplic   = TagCaracter(nome='verAplic'    , codigo='ER04' , tamanho=[1,  20]   , raiz='//retConsSitCTe', namespace=NAMESPACE_CTE, namespace_obrigatorio=False)
        self.cStat      = TagCaracter(nome='cStat'       , codigo='ER05' , tamanho=[1,   3]   , raiz='//retConsSitCTe', namespace=NAMESPACE_CTE, namespace_obrigatorio=False)
        self.xMotivo    = TagCaracter(nome='xMotivo'     , codigo='ER06' , tamanho=[1, 2000]  , raiz='//retConsSitCTe', namespace=NAMESPACE_CTE, namespace_obrigatorio=False)
        self.cUF        = TagInteiro(nome='cUF'          , codigo='ER07' , tamanho=[2,   2, 2], raiz='//retConsSitCTe', namespace=NAMESPACE_CTE, namespace_obrigatorio=False)
        self.protCTe    = None
        self.retCancCTe = None
        self.procEventoCTe = []
        self.caminho_esquema = os.path.join(DIRNAME, 'schema', ESQUEMA_ATUAL + '/')
        self.arquivo_esquema = 'retConsSitCTe_v3.00.xsd'

    def get_xml(self):
        xml = XMLNFe.get_xml(self)
        xml += ABERTURA
        xml += self.versao.xml
        xml += self.tpAmb.xml
        xml += self.verAplic.xml
        xml += self.cStat.xml
        xml += self.xMotivo.xml
        xml += self.cUF.xml

        if self.protCTe is not None:
            xml += self.protCTe.xml

        if self.retCancCTe is not None:
            xml += tira_abertura(self.retCancCTe.xml)

        if not (len(self.procEventoCTe)):
            for procenv in self.procEventoCTe:
                xml += procenv.xml

        xml += '</retConsSitCTe>'
        return xml

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.versao.xml  = arquivo
            self.tpAmb.xml     = arquivo
            self.verAplic.xml  = arquivo
            self.cStat.xml     = arquivo
            self.xMotivo.xml   = arquivo
            self.cUF.xml       = arquivo

            if self._le_noh('//retConsSitCTe/protCTe', ns=NAMESPACE_CTE) is not None:
                self.protCTe = ProtCTe_300()
                self.protCTe.xml = arquivo

            if self._le_noh('//retConsSitCTe/retCancCTe', ns=NAMESPACE_CTE) is not None:
                self.retCancCTe = RetCancCTe_300()
                self.retCancCTe.xml = arquivo

            self.procEventoCTe = self.le_grupo('//retConsSitCTe/procEventoCTe', ProcEventoCTe_300, sigla_ns='cte')

    xml = property(get_xml, set_xml)



# -*- coding: utf-8 -*-
#
# PySPED - Python libraries to deal with Brazil's SPED Project
#
# Copyright (C) 2010-2012
# Copyright (C) Aristides Caldeira <aristides.caldeira at tauga.com.br>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Library General Public License as
# published by the Free Software Foundation, either version 2.1 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# PySPED - Bibliotecas Python para o
#          SPED - Sistema Público de Escrituração Digital
#
# Copyright (C) 2010-2012
# Copyright (C) Aristides Caldeira <aristides.caldeira arroba tauga.com.br>
#
# Este programa é um software livre: você pode redistribuir e/ou modificar
# este programa sob os termos da licença GNU Library General Public License,
# publicada pela Free Software Foundation, em sua versão 2.1 ou, de acordo
# com sua opção, qualquer versão posterior.
#
# Este programa é distribuido na esperança de que venha a ser útil,
# porém SEM QUAISQUER GARANTIAS, nem mesmo a garantia implícita de
# COMERCIABILIDADE ou ADEQUAÇÃO A UMA FINALIDADE ESPECÍFICA. Veja a
# GNU Library General Public License para mais detalhes.
#
# Você deve ter recebido uma cópia da GNU Library General Public License
# juntamente com este programa. Caso esse não seja o caso, acesse:
# <http://www.gnu.org/licenses/>
#

from __future__ import division, print_function, unicode_literals

from pysped.xml_sped import (ABERTURA, NAMESPACE_NFE, TagCaracter,
                             TagDataHora, TagDecimal, TagInteiro, XMLNFe,
                             TagData, TagDataHoraUTC)
from pysped.nfe.leiaute import ESQUEMA_ATUAL_VERSAO_2 as ESQUEMA_ATUAL
import os

DIRNAME = os.path.dirname(__file__)


CONS_NFE_TODAS = '0'
CONS_NFE_SEM_CONFIRMACAO_OPERACAO = '1'
CONS_NFE_SEM_CIENCIA_OPERACAO = '2'

CONS_NFE_EMISSAO_TODOS_EMITENTES = '0'
CONS_NFE_EMISSAO_SOMENTE_TERCEIROS = '1'


class ConsNFeDest(XMLNFe):
    def __init__(self):
        super(ConsNFeDest, self).__init__()
        self.versao    = TagDecimal(nome='consNFeDest', codigo='IP01', propriedade='versao', namespace=NAMESPACE_NFE, valor='1.01', raiz='/')
        self.tpAmb = TagInteiro(nome='tpAmb'   , codigo='IP03', tamanho=[ 1,  1, 1] , raiz='//consNFeDest', valor=2)
        self.xServ = TagCaracter(nome='xServ'  , codigo='IP04', tamanho=[18, 18]    , raiz='//consNFeDest', valor='CONSULTAR NFE DEST')
        self.CNPJ  = TagCaracter(nome='CNPJ'  , codigo='IP05', tamanho=[14, 14], raiz='//consNFeDest')
        self.indNFe = TagInteiro(nome='indNFe'   , codigo='IP06', tamanho=[ 1,  1, 1] , raiz='//consNFeDest', valor=CONS_NFE_TODAS)
        self.indEmi = TagInteiro(nome='indEmi'   , codigo='IP07', tamanho=[ 1,  1, 1] , raiz='//consNFeDest', valor=CONS_NFE_EMISSAO_TODOS_EMITENTES)
        self.ultNSU = TagCaracter(nome='ultNSU'   , codigo='IP08', tamanho=[1, 15], raiz='//consNFeDest', valor='0')
        self.caminho_esquema = os.path.join(DIRNAME, 'schema/', ESQUEMA_ATUAL + '/')
        self.arquivo_esquema = 'consNFeDest_v1.01.xsd'

    def get_xml(self):
        xml = XMLNFe.get_xml(self)
        xml += ABERTURA
        xml += self.versao.xml
        xml += self.tpAmb.xml
        xml += self.xServ.xml
        xml += self.CNPJ.xml
        xml += self.indNFe.xml
        xml += self.indEmi.xml
        xml += self.ultNSU.xml
        xml += '</consNFeDest>'
        return xml

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.versao.xml = arquivo
            self.tpAmb.xml = arquivo
            self.xServ.xml = arquivo
            self.CNPJ.xml = arquivo
            self.indNFe.xml = arquivo
            self.indEmi.xml = arquivo
            self.ultNSU.xml = arquivo

    xml = property(get_xml, set_xml)



class ResNFe(XMLNFe):
    def __init__(self):
        super(ResNFe, self).__init__()
        self.NSU      = TagCaracter(nome='resNFe', propriedade='NSU', raiz='/')
        self.chNFe    = TagCaracter(nome='chNFe' , tamanho=[44, 44], raiz='//resNFe')
        self.CNPJ     = TagCaracter(nome='CNPJ'  , tamanho=[14, 14], raiz='//resNFe', obrigatorio=False)
        self.CPF      = TagCaracter(nome='CPF'   , tamanho=[11, 11], raiz='//resNFe', obrigatorio=False)
        self.xNome    = TagCaracter(nome='xNome' , tamanho=[ 1, 60], raiz='//resNFe')
        self.IE       = TagCaracter(nome='IE'    , tamanho=[ 2, 14], raiz='//resNFe', obrigatorio=False)
        self.dEmi     = TagData(nome='dEmi'      ,                   raiz='//resNFe', obrigatorio=False)
        self.tpNF     = TagCaracter(nome='tpNF'  , tamanho=[ 1,  1], raiz='//resNFe')
        self.vNF      = TagDecimal(nome='vNF'    , tamanho=[1, 15, 1], decimais=[0, 2, 2], raiz='//resNFe')
        self.digVal   = TagCaracter(nome='digVal', tamanho=[28, 28], raiz='//resNFe')
        self.dhRecbto = TagDataHoraUTC(nome='dhRecbto', raiz='//resNFe')
        self.cSitNFe  = TagCaracter(nome='cSitNFe', tamanho=[ 1,  1], raiz='//resNFe')
        self.cSitConf = TagCaracter(nome='cSitConf', tamanho=[ 1,  1], raiz='//resNFe')

    def get_xml(self):
        xml = XMLNFe.get_xml(self)
        xml += '<ret>'
        xml += self.NSU.xml
        xml += self.chNFe.xml
        xml += self.CNPJ.xml
        xml += self.CPF.xml
        xml += self.xNome.xml
        xml += self.IE.xml
        xml += self.dEmi.xml
        xml += self.tpNF.xml
        xml += self.vNF.xml
        xml += self.digVal.xml
        xml += self.dhRecbto.xml
        xml += self.cSitNFe.xml
        xml += self.cSitConf.xml
        xml += '</resNFe>'
        xml += '</ret>'
        return xml

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.NSU.xml   = arquivo
            self.chNFe.xml   = arquivo
            self.CNPJ.xml   = arquivo
            self.CPF.xml   = arquivo
            self.xNome.xml   = arquivo
            self.IE.xml   = arquivo
            self.dEmi.xml   = arquivo
            self.tpNF.xml   = arquivo
            self.vNF.xml   = arquivo
            self.digVal.xml   = arquivo
            self.dhRecbto.xml   = arquivo
            self.cSitNFe.xml   = arquivo
            self.cSitConf.xml   = arquivo

    xml = property(get_xml, set_xml)


class ResCanc(XMLNFe):
    def __init__(self):
        super(ResCanc, self).__init__()
        self.NSU      = TagCaracter(nome='resCanc', propriedade='NSU', raiz='/')
        self.chNFe    = TagCaracter(nome='chNFe' , tamanho=[44, 44], raiz='//resCanc')
        self.CNPJ     = TagCaracter(nome='CNPJ'  , tamanho=[14, 14], raiz='//resCanc', obrigatorio=False)
        self.CPF      = TagCaracter(nome='CPF'   , tamanho=[11, 11], raiz='//resCanc', obrigatorio=False)
        self.xNome    = TagCaracter(nome='xNome' , tamanho=[ 1, 60], raiz='//resCanc')
        self.IE       = TagCaracter(nome='IE'    , tamanho=[ 2, 14], raiz='//resCanc', obrigatorio=False)
        self.dEmi     = TagData(nome='dEmi'      ,                   raiz='//resCanc', obrigatorio=False)
        self.tpNF     = TagCaracter(nome='tpNF'  , tamanho=[ 1,  1], raiz='//resCanc')
        self.vNF      = TagDecimal(nome='vNF'    , tamanho=[1, 15, 1], decimais=[0, 2, 2], raiz='//resCanc')
        self.digVal   = TagCaracter(nome='digVal', tamanho=[28, 28], raiz='//resCanc')
        self.dhRecbto = TagDataHoraUTC(nome='dhRecbto', raiz='//resCanc')
        self.cSitNFe  = TagCaracter(nome='cSitNFe', tamanho=[ 1,  1], raiz='//resCanc')
        self.cSitConf = TagCaracter(nome='cSitConf', tamanho=[ 1,  1], raiz='//resCanc')

    def get_xml(self):
        xml = XMLNFe.get_xml(self)
        xml += '<ret>'
        xml += self.NSU.xml
        xml += self.chNFe.xml
        xml += self.CNPJ.xml
        xml += self.CPF.xml
        xml += self.xNome.xml
        xml += self.IE.xml
        xml += self.dEmi.xml
        xml += self.tpNF.xml
        xml += self.vNF.xml
        xml += self.digVal.xml
        xml += self.dhRecbto.xml
        xml += self.cSitNFe.xml
        xml += self.cSitConf.xml
        xml += '</resCanc>'
        xml += '</ret>'
        return xml

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.NSU.xml   = arquivo
            self.chNFe.xml   = arquivo
            self.CNPJ.xml   = arquivo
            self.CPF.xml   = arquivo
            self.xNome.xml   = arquivo
            self.IE.xml   = arquivo
            self.dEmi.xml   = arquivo
            self.tpNF.xml   = arquivo
            self.vNF.xml   = arquivo
            self.digVal.xml   = arquivo
            self.dhRecbto.xml   = arquivo
            self.cSitNFe.xml   = arquivo
            self.cSitConf.xml   = arquivo

    xml = property(get_xml, set_xml)


class RetConsNFeDest(XMLNFe):
    def __init__(self):
        super(RetConsNFeDest, self).__init__()
        self.versao   = TagDecimal(nome='retConsNFeDest', codigo='IR01', propriedade='versao', namespace=NAMESPACE_NFE, valor='1.01', raiz='/')
        self.tpAmb    = TagInteiro(nome='tpAmb'         , codigo='IR03', tamanho=[1,   1, 1], raiz='//retConsNFeDest')
        self.verAplic = TagCaracter(nome='verAplic'     , codigo='IR04', tamanho=[1,  20]   , raiz='//retConsNFeDest')
        self.cStat    = TagCaracter(nome='cStat'        , codigo='IR05', tamanho=[1,   3]   , raiz='//retConsNFeDest')
        self.xMotivo  = TagCaracter(nome='xMotivo'      , codigo='IR06', tamanho=[1, 255]   , raiz='//retConsNFeDest')
        self.dhResp   = TagDataHora(nome='dhResp'       , codigo='IR07',                      raiz='//retConsNFeDest')
        self.indCont  = TagCaracter(nome='indCont'      , codigo='IR08', tamanho=[1,   1, 1], raiz='//retConsNFeDest', obrigatorio=False)
        self.ultNSU   = TagCaracter(nome='ultNSU'       , codigo='IP09', tamanho=[1, 15]    , raiz='//retConsNFeDest', obrigatorio=False)
        self.resNFe   = []
        self.resCanc  = []
        self.resCCe   = []

        self.caminho_esquema = os.path.join(DIRNAME, 'schema', ESQUEMA_ATUAL + '/')
        self.arquivo_esquema = 'retconsNFeDest_v1.01.xsd'

    def get_xml(self):
        xml = XMLNFe.get_xml(self)
        xml += ABERTURA
        xml += self.versao.xml
        xml += self.tpAmb.xml
        xml += self.verAplic.xml
        xml += self.cStat.xml
        xml += self.xMotivo.xml
        xml += self.dhResp.xml
        xml += self.indCont.xml
        xml += self.ultNSU.xml

        for n in self.resNFe:
            xml += n.xml

        for n in self.resCanc:
            xml += n.xml

        for n in self.resCCe:
            xml += n.xml

        xml += '</retConsNFeDest>'
        return xml

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.versao.xml   = arquivo
            self.tpAmb.xml   = arquivo
            self.verAplic.xml   = arquivo
            self.cStat.xml   = arquivo
            self.xMotivo.xml   = arquivo
            self.dhResp.xml   = arquivo
            self.indCont.xml   = arquivo
            self.ultNSU.xml   = arquivo

            #
            # Técnica para leitura de tags múltiplas
            # As classes dessas tags, e suas filhas, devem ser
            # "reenraizadas" (propriedade raiz) para poderem ser
            # lidas corretamente
            #
            self.resNFe = self.le_grupo('//ret/resNFe', ResNFe)
            self.resCanc = self.le_grupo('//ret/resCanc', ResCanc)
            #self.resCCe = self.le_grupo('//ret/resCCe', ResCCe)

    xml = property(get_xml, set_xml)



# -*- coding: utf-8 -*-

from pysped.xml_sped import *
from pysped.cte.leiaute import ESQUEMA_ATUAL_VERSAO_300 as ESQUEMA_ATUAL
import os

DIRNAME = os.path.dirname(__file__)

class ConsStatServCTe(XMLNFe):
    def __init__(self):
        super(ConsStatServCTe, self).__init__()
        self.versao  = TagDecimal(nome=u'consStatServCte', codigo=u'FP02', propriedade=u'versao', namespace=NAMESPACE_CTE, valor=u'3.00', raiz=u'/')
        self.tpAmb = TagInteiro(nome=u'tpAmb', codigo=u'FP03', tamanho=[1, 1, 1], raiz=u'//consStatServCte', namespace=NAMESPACE_CTE, namespace_obrigatorio=False, valor=2)
        self.xServ = TagCaracter(nome=u'xServ', codigo=u'FP04', tamanho=[6, 6], raiz=u'//consStatServCte', namespace=NAMESPACE_CTE, namespace_obrigatorio=False, valor=u'STATUS')
        self.caminho_esquema = os.path.join(DIRNAME, u'schema/', ESQUEMA_ATUAL + u'/')
        self.arquivo_esquema = u'consStatServCTe_v3.00.xsd'
    
    def get_xml(self):
        xml = XMLNFe.get_xml(self)
        xml += ABERTURA
        xml += self.versao.xml
        xml += self.tpAmb.xml
        xml += self.xServ.xml
        xml += u'</consStatServCte>'
        return xml
        
    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.versao.xml = arquivo
            self.tpAmb.xml  = arquivo
            self.xServ.xml  = arquivo
            
    xml = property(get_xml, set_xml)
    
    
class RetConsStatServCTe(XMLNFe):
    def __init__(self):
        super(RetConsStatServCTe, self).__init__()
        self.versao  = TagDecimal(nome=u'retConsStatServCte', codigo=u'FR02', propriedade=u'versao', namespace=NAMESPACE_CTE, valor=u'3.00', raiz=u'/')
        self.tpAmb = TagInteiro(nome=u'tpAmb', codigo=u'FR03', tamanho=[1, 1, 1], raiz=u'//retConsStatServCte', namespace=NAMESPACE_CTE, namespace_obrigatorio=False, valor=2)
        self.verAplic = TagCaracter(nome=u'verAplic', codigo=u'FR04', tamanho=[1, 20], raiz=u'//retConsStatServCte', namespace=NAMESPACE_CTE, namespace_obrigatorio=False)
        self.cStat = TagInteiro(nome=u'cStat', codigo=u'FR05', tamanho=[3, 3, 3], raiz=u'//retConsStatServCte', namespace=NAMESPACE_CTE, namespace_obrigatorio=False)
        self.xMotivo = TagCaracter(nome=u'xMotivo', codigo=u'FR06', tamanho=[1, 255], raiz=u'//retConsStatServCte', namespace=NAMESPACE_CTE, namespace_obrigatorio=False)
        self.cUF = TagInteiro(nome=u'cUF', codigo=u'FR07', tamanho=[2, 2, 2], raiz=u'//retConsStatServCte', namespace=NAMESPACE_CTE, namespace_obrigatorio=False)
        self.dhRecbto = TagDataHora(nome=u'dhRecbto', codigo=u'FR08', raiz=u'//retConsStatServCte', namespace=NAMESPACE_CTE, namespace_obrigatorio=False)
        self.tMed = TagInteiro(nome=u'tMed', codigo=u'FR09', tamanho=[1, 4], raiz=u'//retConsStatServCte', namespace=NAMESPACE_CTE, namespace_obrigatorio=False, obrigatorio=False)
        self.dhRetorno = TagDataHora(nome='dhRetorno', codigo='FR10', raiz='//retConsStatServCte', namespace=NAMESPACE_CTE, namespace_obrigatorio=False, obrigatorio=False)
        self.xObs = TagCaracter(nome=u'xObs', codigo=u'FR11', tamanho=[1, 255], raiz=u'//retConsStatServCte', namespace=NAMESPACE_CTE, namespace_obrigatorio=False, obrigatorio=False)
        self.caminho_esquema = os.path.join(DIRNAME, u'schema/', ESQUEMA_ATUAL + u'/')
        self.arquivo_esquema = u'consStatServCTe_v3.00.xsd'
    
    def get_xml(self):
        xml = XMLNFe.get_xml(self)
        xml += ABERTURA
        xml += self.versao.xml
        xml += self.tpAmb.xml
        xml += self.verAplic.xml
        xml += self.cStat.xml
        xml += self.xMotivo.xml
        xml += self.cUF.xml
        xml += self.dhRecbto.xml
        xml += self.tMed.xml
        xml += self.dhRetorno.xml
        xml += self.xObs.xml
        xml += u'</retConsStatServCte>'
        return xml
        
    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.versao.xml    = arquivo
            self.tpAmb.xml     = arquivo
            self.verAplic.xml  = arquivo
            self.cStat.xml     = arquivo
            self.xMotivo.xml   = arquivo
            self.cUF.xml       = arquivo
            self.dhRecbto.xml  = arquivo
            self.tMed.xml      = arquivo
            self.dhRetorno.xml = arquivo
            self.xObs.xml      = arquivo
            
    xml = property(get_xml, set_xml)
    


# -*- coding: utf-8 -*-
#
# PySPED - Python libraries to deal with Brazil's SPED Project
#
# Copyright (C) 2010-2012
# Copyright (C) Aristides Caldeira <aristides.caldeira at tauga.com.br>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Library General Public License as
# published by the Free Software Foundation, either version 2.1 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# PySPED - Bibliotecas Python para o
#          SPED - Sistema Público de Escrituração Digital
#
# Copyright (C) 2010-2012
# Copyright (C) Aristides Caldeira <aristides.caldeira arroba tauga.com.br>
#
# Este programa é um software livre: você pode redistribuir e/ou modificar
# este programa sob os termos da licença GNU Library General Public License,
# publicada pela Free Software Foundation, em sua versão 2.1 ou, de acordo
# com sua opção, qualquer versão posterior.
#
# Este programa é distribuido na esperança de que venha a ser útil,
# porém SEM QUAISQUER GARANTIAS, nem mesmo a garantia implícita de
# COMERCIABILIDADE ou ADEQUAÇÃO A UMA FINALIDADE ESPECÍFICA. Veja a
# GNU Library General Public License para mais detalhes.
#
# Você deve ter recebido uma cópia da GNU Library General Public License
# juntamente com este programa. Caso esse não seja o caso, acesse:
# <http://www.gnu.org/licenses/>
#

from __future__ import division, print_function, unicode_literals

#
# Versão 2.00, usada a partir de maio/2010
#
#

#
# O esquemar pl_006m contém os esquemas dos enventos da NF-e
#
from pysped.nfe.leiaute import ESQUEMA_ATUAL_VERSAO_2 as ESQUEMA_ATUAL

#
# Envelopes SOAP
#
from pysped.nfe.leiaute.soap_200 import SOAPEnvio as SOAPEnvio_200
from pysped.nfe.leiaute.soap_200 import SOAPRetorno as SOAPRetorno_200

#
# Emissão de NF-e
#
from pysped.nfe.leiaute.nfe_200 import NFe as NFe_200
from pysped.nfe.leiaute.nfe_200 import NFRef as NFRef_200
from pysped.nfe.leiaute.nfe_200 import Det as Det_200
from pysped.nfe.leiaute.nfe_200 import DI as DI_200
from pysped.nfe.leiaute.nfe_200 import Adi as Adi_200
from pysped.nfe.leiaute.nfe_200 import Med as Med_200
from pysped.nfe.leiaute.nfe_200 import Arma as Arma_200
from pysped.nfe.leiaute.nfe_200 import Reboque as Reboque_200
from pysped.nfe.leiaute.nfe_200 import Vol as Vol_200
from pysped.nfe.leiaute.nfe_200 import Lacres as Lacres_200
from pysped.nfe.leiaute.nfe_200 import Dup as Dup_200
from pysped.nfe.leiaute.nfe_200 import ObsCont as ObsCont_200
from pysped.nfe.leiaute.nfe_200 import ObsFisco as ObsFisco_200
from pysped.nfe.leiaute.nfe_200 import ProcRef as ProcRef_200

#
# Envio de lote de NF-e
#
from pysped.nfe.leiaute.envinfe_200 import EnviNFe as EnviNFe_200
from pysped.nfe.leiaute.envinfe_200 import RetEnviNFe as RetEnviNFe_200

#
# Consulta do recibo do lote de NF-e
#
from pysped.nfe.leiaute.consrecinfe_200 import ConsReciNFe as ConsReciNFe_200
from pysped.nfe.leiaute.consrecinfe_200 import RetConsReciNFe as RetConsReciNFe_200
from pysped.nfe.leiaute.consrecinfe_200 import ProtNFe as ProtNFe_200
from pysped.nfe.leiaute.consrecinfe_200 import ProcNFe as ProcNFe_200

#
# Cancelamento de NF-e
#
from pysped.nfe.leiaute.cancnfe_200 import CancNFe as CancNFe_200
from pysped.nfe.leiaute.cancnfe_200 import RetCancNFe as RetCancNFe_200
from pysped.nfe.leiaute.cancnfe_200 import ProcCancNFe as ProcCancNFe_200

#
# Inutilização de NF-e
#
from pysped.nfe.leiaute.inutnfe_200 import InutNFe as InutNFe_200
from pysped.nfe.leiaute.inutnfe_200 import RetInutNFe as RetInutNFe_200
from pysped.nfe.leiaute.inutnfe_200 import ProcInutNFe as ProcInutNFe_200

#
# Consulta a situação de NF-e
#
from pysped.nfe.leiaute.conssitnfe_200 import ConsSitNFe as ConsSitNFe_200
from pysped.nfe.leiaute.conssitnfe_200 import RetConsSitNFe as RetConsSitNFe_200

#
# Consulta a situação do serviço
#
from pysped.nfe.leiaute.consstatserv_200 import ConsStatServ as ConsStatServ_200
from pysped.nfe.leiaute.consstatserv_200 import RetConsStatServ as RetConsStatServ_200

#
# Consulta cadastro
#
#from pysped.nfe.leiaute.conscad_101 import ConsCad as ConsCad_101
#from pysped.nfe.leiaute.conscad_101 import RetConsCad as RetConsCad_101

#
# Eventos da NF-e - Carta de Correção Eletrônica
#
from pysped.nfe.leiaute.evtccenfe_100 import EventoCCe as EventoCCe_100
from pysped.nfe.leiaute.evtccenfe_100 import RetEventoCCe as RetEventoCCe_100
from pysped.nfe.leiaute.evtccenfe_100 import ProcEventoCCe as ProcEventoCCe_100
from pysped.nfe.leiaute.evtccenfe_100 import EnvEventoCCe as EnvEventoCCe_100
from pysped.nfe.leiaute.evtccenfe_100 import RetEnvEventoCCe as RetEnvEventoCCe_100

#
# Eventos da NF-e - Cancelamento como Evento
#
from pysped.nfe.leiaute.evtcancnfe_100 import EventoCancNFe as EventoCancNFe_100
from pysped.nfe.leiaute.evtcancnfe_100 import RetEventoCancNFe as RetEventoCancNFe_100
from pysped.nfe.leiaute.evtcancnfe_100 import ProcEventoCancNFe as ProcEventoCancNFe_100
from pysped.nfe.leiaute.evtcancnfe_100 import EnvEventoCancNFe as EnvEventoCancNFe_100
from pysped.nfe.leiaute.evtcancnfe_100 import RetEnvEventoCancNFe as RetEnvEventoCancNFe_100

#
# Eventos da NF-e - Confirmação de Recebimento/Manifestação do Destinatário
#
from pysped.nfe.leiaute.evtconfrecebimento_100 import EventoConfRecebimento as EventoConfRecebimento_100
from pysped.nfe.leiaute.evtconfrecebimento_100 import RetEventoConfRecebimento as RetEventoConfRecebimento_100
from pysped.nfe.leiaute.evtconfrecebimento_100 import ProcEventoConfRecebimento as ProcEventoConfRecebimento_100
from pysped.nfe.leiaute.evtconfrecebimento_100 import EnvEventoConfRecebimento as EnvEventoConfRecebimento_100
from pysped.nfe.leiaute.evtconfrecebimento_100 import RetEnvEventoConfRecebimento as RetEnvEventoConfRecebimento_100
from pysped.nfe.leiaute.evtconfrecebimento_100 import CONF_RECEBIMENTO_CONFIRMAR_OPERACAO
from pysped.nfe.leiaute.evtconfrecebimento_100 import CONF_RECEBIMENTO_CIENCIA_OPERACAO
from pysped.nfe.leiaute.evtconfrecebimento_100 import CONF_RECEBIMENTO_DESCONHECIMENTO_OPERACAO
from pysped.nfe.leiaute.evtconfrecebimento_100 import CONF_RECEBIMENTO_OPERACAO_NAO_REALIZADA
from pysped.nfe.leiaute.evtconfrecebimento_100 import DESCEVENTO_CONF_RECEBIMENTO

# Pyflakes

Adi_200
Arma_200
CONF_RECEBIMENTO_CIENCIA_OPERACAO
CONF_RECEBIMENTO_CONFIRMAR_OPERACAO
CONF_RECEBIMENTO_DESCONHECIMENTO_OPERACAO
CONF_RECEBIMENTO_OPERACAO_NAO_REALIZADA
CancNFe_200
ConsReciNFe_200
ConsSitNFe_200
ConsStatServ_200
DESCEVENTO_CONF_RECEBIMENTO
DI_200
Det_200
Dup_200
ESQUEMA_ATUAL
EnvEventoCCe_100
EnvEventoCancNFe_100
EnvEventoConfRecebimento_100
EnviNFe_200
EventoCCe_100
EventoCancNFe_100
EventoConfRecebimento_100
InutNFe_200
Lacres_200
Med_200
NFRef_200
NFe_200
ObsCont_200
ObsFisco_200
ProcCancNFe_200
ProcEventoCCe_100
ProcEventoCancNFe_100
ProcEventoConfRecebimento_100
ProcInutNFe_200
ProcNFe_200
ProcRef_200
ProtNFe_200
Reboque_200
RetCancNFe_200
RetConsReciNFe_200
RetConsSitNFe_200
RetConsStatServ_200
RetEnvEventoCCe_100
RetEnvEventoCancNFe_100
RetEnvEventoConfRecebimento_100
RetEnviNFe_200
RetEventoCCe_100
RetEventoCancNFe_100
RetEventoConfRecebimento_100
RetInutNFe_200
SOAPEnvio_200
SOAPRetorno_200
Vol_200



# -*- coding: utf-8 -*-
#
# PySPED - Python libraries to deal with Brazil's SPED Project
#
# Copyright (C) 2010-2012
# Copyright (C) Aristides Caldeira <aristides.caldeira at tauga.com.br>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Library General Public License as
# published by the Free Software Foundation, either version 2.1 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# PySPED - Bibliotecas Python para o
#          SPED - Sistema Público de Escrituração Digital
#
# Copyright (C) 2010-2012
# Copyright (C) Aristides Caldeira <aristides.caldeira arroba tauga.com.br>
#
# Este programa é um software livre: você pode redistribuir e/ou modificar
# este programa sob os termos da licença GNU Library General Public License,
# publicada pela Free Software Foundation, em sua versão 2.1 ou, de acordo
# com sua opção, qualquer versão posterior.
#
# Este programa é distribuido na esperança de que venha a ser útil,
# porém SEM QUAISQUER GARANTIAS, nem mesmo a garantia implícita de
# COMERCIABILIDADE ou ADEQUAÇÃO A UMA FINALIDADE ESPECÍFICA. Veja a
# GNU Library General Public License para mais detalhes.
#
# Você deve ter recebido uma cópia da GNU Library General Public License
# juntamente com este programa. Caso esse não seja o caso, acesse:
# <http://www.gnu.org/licenses/>
#

from __future__ import division, print_function, unicode_literals

from .webservices_flags import (NFE_AMBIENTE_PRODUCAO,
                                NFE_AMBIENTE_HOMOLOGACAO,
                                WS_DPEC_CONSULTA,
                                WS_DPEC_RECEPCAO,
                                WS_NFE_AUTORIZACAO,
                                WS_NFE_CONSULTA,
                                WS_NFE_CONSULTA_AUTORIZACAO,
                                WS_NFE_CONSULTA_CADASTRO,
                                WS_NFE_CONSULTA_DESTINADAS,
                                WS_NFE_DOWNLOAD,
                                WS_NFE_INUTILIZACAO,
                                WS_NFE_SITUACAO,
                                WS_NFE_RECEPCAO_EVENTO,
                                WS_DFE_DISTRIBUICAO)

import webservices_3


SVRS = {
    # o servidor da consulta de cadastro é diferente dos demais...
    NFE_AMBIENTE_PRODUCAO: {
        'servidor'             : 'nfce.svrs.rs.gov.br',
        'servidor%s' % WS_NFE_CONSULTA_CADASTRO: 'cad.svrs.rs.gov.br',
        WS_NFE_RECEPCAO_EVENTO: 'ws/recepcaoevento/recepcaoevento.asmx',
        WS_NFE_AUTORIZACAO      : 'ws/NfeAutorizacao/NfeAutorizacao.asmx',
        WS_NFE_CONSULTA_AUTORIZACAO : 'ws/NfeRetAutorizacao/NfeRetAutorizacao.asmx',
        WS_NFE_CONSULTA_CADASTRO: 'ws/CadConsultaCadastro/CadConsultaCadastro2.asmx',
        WS_NFE_INUTILIZACAO    : 'ws/nfeinutilizacao/nfeinutilizacao2.asmx',
        WS_NFE_CONSULTA        : 'ws/NfeConsulta/NfeConsulta2.asmx',
        WS_NFE_SITUACAO        : 'ws/NfeStatusServico/NfeStatusServico2.asmx',
    },
    NFE_AMBIENTE_HOMOLOGACAO: {
        'servidor'             : 'nfce-homologacao.svrs.rs.gov.br',
        WS_NFE_RECEPCAO_EVENTO: 'ws/recepcaoevento/recepcaoevento.asmx',
        WS_NFE_AUTORIZACAO      : 'ws/NfeAutorizacao/NfeAutorizacao.asmx',
        WS_NFE_CONSULTA_AUTORIZACAO : 'ws/NfeRetAutorizacao/NfeRetAutorizacao.asmx',
        WS_NFE_CONSULTA_CADASTRO: 'ws/CadConsultaCadastro/CadConsultaCadastro2.asmx',
        WS_NFE_INUTILIZACAO    : 'ws/nfeinutilizacao/nfeinutilizacao2.asmx',
        WS_NFE_CONSULTA        : 'ws/NfeConsulta/NfeConsulta2.asmx',
        WS_NFE_SITUACAO        : 'ws/NfeStatusServico/NfeStatusServico2.asmx',
    }
}


UFAM = webservices_3.UFAM
UFGO = webservices_3.UFGO
UFMT = webservices_3.UFMT
UFMS = webservices_3.UFMS
UFPR = webservices_3.UFPR
UFPE = webservices_3.UFPE
UFRS = webservices_3.UFRS
UFSP = webservices_3.UFSP


#
# Informação obtida em
# http://www.nfe.fazenda.gov.br/portal/webServices.aspx
#  Última verificação: 15/08/2014 16:22
#
# UF que utilizam a SVAN - Sefaz Virtual do Ambiente Nacional: MA, PA, PI
# UF que utilizam a SVRS - Sefaz Virtual do RS:
# - Para serviço de Consulta Cadastro: AC, RN, PB, SC
# - Para demais serviços relacionados com o sistema da NF-e: AC, AL, AP, DF, PB, RJ, RN, RO, RR, SC, SE, TO
# Autorizadores: AM BA CE GO MG MA MS MT PE PR RS SP
#

ESTADO_WS = {
    'AC': SVRS,
    'AL': SVRS,
    'AM': UFAM,
    'AP': SVRS,
    'BA': SVRS,
    'CE': SVRS,
    'DF': SVRS,
    'ES': SVRS,
    'GO': UFGO,
    'MA': SVRS,
    'MG': UFRS,
    'MS': UFMS,
    'MT': UFMT,
    'PA': SVRS,
    'PB': SVRS,
    'PE': UFPE,
    'PI': SVRS,
    'PR': UFPR,
    'RJ': SVRS,
    'RN': SVRS,
    'RO': SVRS,
    'RR': SVRS,
    'RS': UFRS,
    'SC': SVRS,
    'SE': SVRS,
    'SP': UFSP,
    'TO': SVRS,
}

ESTADO_QRCODE = {
    'AC': {
        NFE_AMBIENTE_PRODUCAO: 'http://www.sefaznet.ac.gov.br/nfce/qrcode',
        NFE_AMBIENTE_HOMOLOGACAO: 'http://hml.sefaznet.ac.gov.br/nfce/qrcode',
    },
    'AL': {
        NFE_AMBIENTE_PRODUCAO: 'http://nfce.sefaz.al.gov.br/QRCode/consultarNFCe.jsp',
        NFE_AMBIENTE_HOMOLOGACAO: 'http://nfce.sefaz.al.gov.br/QRCode/consultarNFCe.jsp',
    },
    'AM': {
        NFE_AMBIENTE_PRODUCAO: 'http://sistemas.sefaz.am.gov.br/nfceweb/consultarNFCe.jsp',
        NFE_AMBIENTE_HOMOLOGACAO: 'http://homnfce.sefaz.am.gov.br/nfceweb/consultarNFCe.jsp',
    },
    'AP': {
        NFE_AMBIENTE_PRODUCAO: 'https://www.sefaz.ap.gov.br/nfce/nfcep.php',
        NFE_AMBIENTE_HOMOLOGACAO: 'https://www.sefaz.ap.gov.br/nfcehml/nfce.php',
    },
    'BA': {
        NFE_AMBIENTE_PRODUCAO: 'http://nfe.sefaz.ba.gov.br/servicos/nfce/modulos/geral/NFCEC_consulta_chave_acesso.aspx',
        NFE_AMBIENTE_HOMOLOGACAO: 'http://hnfe.sefaz.ba.gov.br/servicos/nfce/modulos/geral/NFCEC_consulta_chave_acesso.aspx',
    },
    'CE': {
        NFE_AMBIENTE_PRODUCAO: '',
        NFE_AMBIENTE_HOMOLOGACAO: 'http://nfceh.sefaz.ce.gov.br/pages/ShowNFCe.html',
    },
    'DF': {
        NFE_AMBIENTE_PRODUCAO: 'http://dec.fazenda.df.gov.br/ConsultarNFCe.aspx',
        NFE_AMBIENTE_HOMOLOGACAO: 'http://dec.fazenda.df.gov.br/ConsultarNFCe.aspx',
    },
    'ES': {
        NFE_AMBIENTE_PRODUCAO: '',
        NFE_AMBIENTE_HOMOLOGACAO: 'http://homologacao.sefaz.es.gov.br/ConsultaNFCe/qrcode.aspx',
    },
    'GO': {
        NFE_AMBIENTE_PRODUCAO: 'http://nfe.sefaz.go.gov.br/nfeweb/sites/nfce/danfeNFCe',
        NFE_AMBIENTE_HOMOLOGACAO: 'http://homolog.sefaz.go.gov.br/nfeweb/sites/nfce/danfeNFCe',
    },
    'MA': {
        NFE_AMBIENTE_PRODUCAO: 'http://www.nfce.sefaz.ma.gov.br/portal/consultarNFCe.jsp',
        NFE_AMBIENTE_HOMOLOGACAO: 'http://www.hom.nfce.sefaz.ma.gov.br/portal/consultarNFCe.jsp',
    },
    'MG': {
        NFE_AMBIENTE_PRODUCAO: '',
        NFE_AMBIENTE_HOMOLOGACAO: '',
    },
    'MS': {
        NFE_AMBIENTE_PRODUCAO: 'http://www.dfe.ms.gov.br/nfce/qrcode',
        NFE_AMBIENTE_HOMOLOGACAO: 'http://www.dfe.ms.gov.br/nfce/qrcode',
    },
    'MT': {
        NFE_AMBIENTE_PRODUCAO: 'http://www.sefaz.mt.gov.br/nfce/consultanfce',
        NFE_AMBIENTE_HOMOLOGACAO: 'http://homologacao.sefaz.mt.gov.br/nfce/consultanfce',
    },
    'PA': {
        NFE_AMBIENTE_PRODUCAO: 'https://appnfc.sefa.pa.gov.br/portal/view/consultas/nfce/nfceForm.seam',
        NFE_AMBIENTE_HOMOLOGACAO: 'https://appnfc.sefa.pa.gov.br/portal-homologacao/view/consultas/nfce/nfceForm.seam',
    },
    'PB': {
        NFE_AMBIENTE_PRODUCAO: 'http://www.receita.pb.gov.br/nfce',
        NFE_AMBIENTE_HOMOLOGACAO: 'http://www.receita.pb.gov.br/nfcehom',
    },
    'PE': {
        NFE_AMBIENTE_PRODUCAO: 'http://nfce.sefaz.pe.gov.br/nfce-web/consultarNFCe',
        NFE_AMBIENTE_HOMOLOGACAO: 'http://nfcehomolog.sefaz.pe.gov.br/nfce-web/consultarNFCe',
    },
    'PI': {
        NFE_AMBIENTE_PRODUCAO: 'http://webas.sefaz.pi.gov.br/nfceweb/consultarNFCe.jsf',
        NFE_AMBIENTE_HOMOLOGACAO: 'http://webas.sefaz.pi.gov.br/nfceweb-homologacao/consultarNFCe.jsf',
    },
    'PR': {
        NFE_AMBIENTE_PRODUCAO: 'http://www.dfeportal.fazenda.pr.gov.br/dfe-portal/rest/servico/consultaNFCe',
        NFE_AMBIENTE_HOMOLOGACAO: 'http://www.dfeportal.fazenda.pr.gov.br/dfe-portal/rest/servico/consultaNFCe',
    },
    'RJ': {
        NFE_AMBIENTE_PRODUCAO: 'http://www4.fazenda.rj.gov.br/consultaNFCe/QRCode',
        NFE_AMBIENTE_HOMOLOGACAO: 'http://www4.fazenda.rj.gov.br/consultaNFCe/QRCode',
    },
    'RN': {
        NFE_AMBIENTE_PRODUCAO: 'http://nfce.set.rn.gov.br/consultarNFCe.aspx',
        NFE_AMBIENTE_HOMOLOGACAO: 'http://hom.nfce.set.rn.gov.br/consultarNFCe.aspx',
    },
    'RO': {
        NFE_AMBIENTE_PRODUCAO: 'http://www.nfce.sefin.ro.gov.br/consultanfce/consulta.jsp',
        NFE_AMBIENTE_HOMOLOGACAO: 'http://www.nfce.sefin.ro.gov.br/consultanfce/consulta.jsp',
    },
    'RR': {
        NFE_AMBIENTE_PRODUCAO: 'https://www.sefaz.rr.gov.br/nfce/servlet/qrcode',
        NFE_AMBIENTE_HOMOLOGACAO: 'http://200.174.88.103:8080/nfce/servlet/qrcode',
    },
    'RS': {
        NFE_AMBIENTE_PRODUCAO: 'https://www.sefaz.rs.gov.br/NFCE/NFCE-COM.aspx',
        NFE_AMBIENTE_HOMOLOGACAO: 'https://www.sefaz.rs.gov.br/NFCE/NFCE-COM.aspx',
    },
    'SC': {
        NFE_AMBIENTE_PRODUCAO: '',
        NFE_AMBIENTE_HOMOLOGACAO: '',
    },
    'SE': {
        NFE_AMBIENTE_PRODUCAO: 'http://www.nfce.se.gov.br/portal/consultarNFCe.jsp',
        NFE_AMBIENTE_HOMOLOGACAO: 'http://www.hom.nfe.se.gov.br/portal/consultarNFCe.jsp',
    },
    'SP': {
        NFE_AMBIENTE_PRODUCAO: 'https://www.nfce.fazenda.sp.gov.br/NFCeConsultaPublica/Paginas/ConsultaQRCode.aspx',
        NFE_AMBIENTE_HOMOLOGACAO: 'https://www.homologacao.nfce.fazenda.sp.gov.br/NFCeConsultaPublica/Paginas/ConsultaQRCode.aspx',
    },
    'TO': {
        NFE_AMBIENTE_PRODUCAO: '',
        NFE_AMBIENTE_HOMOLOGACAO: '',
    },
}

ESTADO_CONSULTA_NFCE = {
    'AC': {
        NFE_AMBIENTE_PRODUCAO: 'http://www.sefaznet.ac.gov.br/nfce/',
        NFE_AMBIENTE_HOMOLOGACAO: 'http://hml.sefaznet.ac.gov.br/nfce/',
    },
    'AL': {
        NFE_AMBIENTE_PRODUCAO: 'http://nfce.sefaz.al.gov.br/consultaNFCe.htm',
        NFE_AMBIENTE_HOMOLOGACAO: 'http://nfce.sefaz.al.gov.br/consultaNFCe.htm',
    },
    'AM': {
        NFE_AMBIENTE_PRODUCAO: 'http://sistemas.sefaz.am.gov.br/nfceweb/formConsulta.do',
        NFE_AMBIENTE_HOMOLOGACAO: 'http://homnfce.sefaz.am.gov.br/nfceweb/formConsulta.do',
    },
    'AP': {
        NFE_AMBIENTE_PRODUCAO: 'https://www.sefaz.ap.gov.br/sate/seg/SEGf_AcessarFuncao.jsp?cdFuncao=FIS_1261',
        NFE_AMBIENTE_HOMOLOGACAO: 'https://www.sefaz.ap.gov.br/nfcehml/nfce.php',
    },
    'BA': {
        NFE_AMBIENTE_PRODUCAO: 'http://nfe.sefaz.ba.gov.br/servicos/nfce/default.aspx',
        NFE_AMBIENTE_HOMOLOGACAO: 'http://nfe.sefaz.ba.gov.br/servicos/nfce/default.aspx',
    },
    'CE': {
        NFE_AMBIENTE_PRODUCAO: '',
        NFE_AMBIENTE_HOMOLOGACAO: 'http://nfceh.sefaz.ce.gov.br/pages/consultaChaveAcesso.jsf',
    },
    'DF': {
        NFE_AMBIENTE_PRODUCAO: 'http://dec.fazenda.df.gov.br/nfce',
        NFE_AMBIENTE_HOMOLOGACAO: 'http://dec.fazenda.df.gov.br/nfce',
    },
    'ES': {
        NFE_AMBIENTE_PRODUCAO: '',
        NFE_AMBIENTE_HOMOLOGACAO: 'http://app.sefaz.es.gov.br/ConsultaNFCe',
    },
    'GO': {
        NFE_AMBIENTE_PRODUCAO: 'http://nfe.sefaz.go.gov.br/nfeweb/jsp/ConsultaDANFENFCe.jsf',
        NFE_AMBIENTE_HOMOLOGACAO: 'http://homolog.sefaz.go.gov.br/nfeweb/jsp/ConsultaDANFENFCe.jsf',
    },
    'MA': {
        NFE_AMBIENTE_PRODUCAO: 'http://www.nfce.sefaz.ma.gov.br/portal/consultaNFe.do',
        NFE_AMBIENTE_HOMOLOGACAO: 'http://www.hom.nfce.sefaz.ma.gov.br/portal/consultaNFe.do',
    },
    'MG': {
        NFE_AMBIENTE_PRODUCAO: '',
        NFE_AMBIENTE_HOMOLOGACAO: '',
    },
    'MS': {
        NFE_AMBIENTE_PRODUCAO: 'http://www.dfe.ms.gov.br/nfce',
        NFE_AMBIENTE_HOMOLOGACAO: 'http://www.dfe.ms.gov.br/nfce',
    },
    'MT': {
        NFE_AMBIENTE_PRODUCAO: 'http://www.sefaz.mt.gov.br/nfce/consultanfce',
        NFE_AMBIENTE_HOMOLOGACAO: 'http://homologacao.sefaz.mt.gov.br/nfce/consultanfce',
    },
    'PA': {
        NFE_AMBIENTE_PRODUCAO: 'https://appnfc.sefa.pa.gov.br/portal/view/consultas/nfce/consultanfce.seam',
        NFE_AMBIENTE_HOMOLOGACAO: 'https://appnfc.sefa.pa.gov.br/portal-homologacao/view/consultas/nfce/consultanfce.seam',
    },
    'PB': {
        NFE_AMBIENTE_PRODUCAO: 'http://www.receita.pb.gov.br/nfce',
        NFE_AMBIENTE_HOMOLOGACAO: 'http://www.receita.pb.gov.br/nfcehom',
    },
    'PE': {
        NFE_AMBIENTE_PRODUCAO: 'http://nfce.sefaz.pe.gov.br/nfce-web/consultarNFCe',
        NFE_AMBIENTE_HOMOLOGACAO: 'http://nfcehomolog.sefaz.pe.gov.br/nfce-web/consultarNFCe',
    },
    'PI': {
        NFE_AMBIENTE_PRODUCAO: '',
        NFE_AMBIENTE_HOMOLOGACAO: '',
    },
    'PR': {
        NFE_AMBIENTE_PRODUCAO: 'http://www.fazenda.pr.gov.br/',
        NFE_AMBIENTE_HOMOLOGACAO: 'http://www.fazenda.pr.gov.br/',
    },
    'RJ': {
        NFE_AMBIENTE_PRODUCAO: 'http://nfce.fazenda.rj.gov.br/consulta',
        NFE_AMBIENTE_HOMOLOGACAO: 'http://nfce.fazenda.rj.gov.br/consulta',
    },
    'RN': {
        NFE_AMBIENTE_PRODUCAO: 'http://nfce.set.rn.gov.br/portalDFE/NFCe/ConsultaNFCe.aspx',
        NFE_AMBIENTE_HOMOLOGACAO: 'http://hom.nfce.set.rn.gov.br/portalDFE/NFCe/ConsultaNFCe.aspx',
    },
    'RO': {
        NFE_AMBIENTE_PRODUCAO: 'http://www.nfce.sefin.ro.gov.br',
        NFE_AMBIENTE_HOMOLOGACAO: 'http://www.nfce.sefin.ro.gov.br/consultaAmbHomologacao.jsp',
    },
    'RR': {
        NFE_AMBIENTE_PRODUCAO: 'https://www.sefaz.rr.gov.br/nfce/servlet/wp_consulta_nfce',
        NFE_AMBIENTE_HOMOLOGACAO: 'http://200.174.88.103:8080/nfce/servlet/wp_consulta_nfce',
    },
    'RS': {
        NFE_AMBIENTE_PRODUCAO: 'https://www.sefaz.rs.gov.br/NFE/NFE-NFC.aspx',
        NFE_AMBIENTE_HOMOLOGACAO: 'https://www.sefaz.rs.gov.br/NFE/NFE-NFC.aspx',
    },
    'SC': {
        NFE_AMBIENTE_PRODUCAO: '',
        NFE_AMBIENTE_HOMOLOGACAO: '',
    },
    'SE': {
        NFE_AMBIENTE_PRODUCAO: '',
        NFE_AMBIENTE_HOMOLOGACAO: '',
    },
    'SP': {
        NFE_AMBIENTE_PRODUCAO: 'https://www.nfce.fazenda.sp.gov.br/NFCeConsultaPublica/Paginas/ConsultaPublica.aspx',
        NFE_AMBIENTE_HOMOLOGACAO: 'https://www.homologacao.nfce.fazenda.sp.gov.br/NFCeConsultaPublica/Paginas/ConsultaPublica.aspx',
    },
    'TO': {
        NFE_AMBIENTE_PRODUCAO: '',
        NFE_AMBIENTE_HOMOLOGACAO: '',
    },
}




# -*- coding: utf-8 -*-
#
# PySPED - Python libraries to deal with Brazil's SPED Project
#
# Copyright (C) 2010-2012
# Copyright (C) Aristides Caldeira <aristides.caldeira at tauga.com.br>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Library General Public License as
# published by the Free Software Foundation, either version 2.1 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# PySPED - Bibliotecas Python para o
#          SPED - Sistema Público de Escrituração Digital
#
# Copyright (C) 2010-2012
# Copyright (C) Aristides Caldeira <aristides.caldeira arroba tauga.com.br>
#
# Este programa é um software livre: você pode redistribuir e/ou modificar
# este programa sob os termos da licença GNU Library General Public License,
# publicada pela Free Software Foundation, em sua versão 2.1 ou, de acordo
# com sua opção, qualquer versão posterior.
#
# Este programa é distribuido na esperança de que venha a ser útil,
# porém SEM QUAISQUER GARANTIAS, nem mesmo a garantia implícita de
# COMERCIABILIDADE ou ADEQUAÇÃO A UMA FINALIDADE ESPECÍFICA. Veja a
# GNU Library General Public License para mais detalhes.
#
# Você deve ter recebido uma cópia da GNU Library General Public License
# juntamente com este programa. Caso esse não seja o caso, acesse:
# <http://www.gnu.org/licenses/>
#

from __future__ import division, print_function, unicode_literals

from os.path import abspath, dirname
from datetime import datetime
from pysped.nfe import ProcessadorNFe
from pysped.nfe.webservices_flags import (Det_200,
                                          NFe_200,
                                          UF_CODIGO,
                                          WS_NFE_CONSULTA_RECIBO)


FILE_DIR = abspath(dirname(__file__))


if __name__ == '__main__':
    p = ProcessadorNFe()
    p.versao              = '2.00'
    p.estado              = 'SP'
    #p.certificado.arquivo = 'certificado.pfx'
    #p.certificado.senha   = 'senha'

    #
    # arquivo 'certificado_caminho.txt' deve conter o caminho para o 'certificado.pfx'
    #
    p.certificado.arquivo = open(FILE_DIR+'/certificado_caminho.txt').read().strip()

    #
    # arquivo 'certificado_senha.txt' deve conter a senha para o 'certificado.pfx'
    #
    p.certificado.senha   = open(FILE_DIR+'/certificado_senha.txt').read().strip()

    p.salva_arquivos      = True
    p.contingencia_SCAN   = False
    p.caminho = ''

    #
    # Instancia uma NF-e
    #
    n = NFe_200()

    #
    # Identificação da NF-e
    #
    n.infNFe.ide.cUF.valor     = UF_CODIGO['SP']
    n.infNFe.ide.natOp.valor   = 'Venda de produto do estabelecimento'
    n.infNFe.ide.indPag.valor  = 2
    n.infNFe.ide.serie.valor   = 101
    n.infNFe.ide.nNF.valor     = 27
    n.infNFe.ide.dEmi.valor    = datetime(2010, 4, 12)
    n.infNFe.ide.dSaiEnt.valor = datetime(2010, 4, 12)
    n.infNFe.ide.cMunFG.valor  = 3513801
    n.infNFe.ide.tpImp.valor   = 1
    n.infNFe.ide.tpEmis.valor  = 1
    n.infNFe.ide.indPag.valor  = 1
    n.infNFe.ide.finNFe.valor  = 1
    n.infNFe.ide.procEmi.valor = 0
    n.infNFe.ide.verProc.valor = 'PySPED NF-e'

    #
    # Emitente
    #
    n.infNFe.emit.CNPJ.valor  = '11111111111111'
    n.infNFe.emit.xNome.valor = 'Razão Social Ltda. EPP'
    n.infNFe.emit.xFant.valor = 'Nome Fantasia'
    n.infNFe.emit.enderEmit.xLgr.valor    = 'Al. Kenworthy'
    n.infNFe.emit.enderEmit.nro.valor     = '140'
    n.infNFe.emit.enderEmit.xCpl.valor    = ''
    n.infNFe.emit.enderEmit.xBairro.valor = 'Jd. Santa Rosália'
    n.infNFe.emit.enderEmit.cMun.valor    = '3552205'
    n.infNFe.emit.enderEmit.xMun.valor    = 'Sorocaba'
    n.infNFe.emit.enderEmit.UF.valor      = 'SP'
    n.infNFe.emit.enderEmit.CEP.valor     = '18095360'
    #n.infNFe.emit.enderEmit.cPais.valor   = '1058'
    #n.infNFe.emit.enderEmit.xPais.valor   = 'Brasil'
    n.infNFe.emit.enderEmit.fone.valor    = '1534110602'
    n.infNFe.emit.IE.valor = '111111111111'
    #
    # Regime tributário
    #
    n.infNFe.emit.CRT.valor = 3

    #
    # Destinatário
    #
    n.infNFe.dest.CNPJ.valor  = '11111111111111'
    n.infNFe.dest.xNome.valor = 'Razão Social Ltda. EPP'
    n.infNFe.dest.enderDest.xLgr.valor    = 'Al. Kenworthy'
    n.infNFe.dest.enderDest.nro.valor     = '140'
    n.infNFe.dest.enderDest.xCpl.valor    = ''
    n.infNFe.dest.enderDest.xBairro.valor = 'Jd. Santa Rosália'
    n.infNFe.dest.enderDest.cMun.valor    = '3552205'
    n.infNFe.dest.enderDest.xMun.valor    = 'Sorocaba'
    n.infNFe.dest.enderDest.UF.valor      = 'SP'
    n.infNFe.dest.enderDest.CEP.valor     = '18095360'
    #n.infNFe.dest.enderDest.cPais.valor   = '1058'
    #n.infNFe.dest.enderDest.xPais.valor   = 'Brasil'
    n.infNFe.dest.enderDest.fone.valor    = '1534110602'
    n.infNFe.dest.IE.valor = '111111111111'
    #
    # Emeio
    #
    n.infNFe.dest.email.valor = 'emeio@servidor.com.br'

    #
    # Detalhe
    #
    d1 = Det_200()

    d1.nItem.valor = 1
    d1.prod.cProd.valor    = 'código do produto'
    d1.prod.cEAN.valor     = ''
    d1.prod.xProd.valor    = 'Descrição do produto'
    d1.prod.NCM.valor      = '01'
    d1.prod.EXTIPI.valor   = ''
    d1.prod.CFOP.valor     = '5101'
    d1.prod.uCom.valor     = 'UN'
    d1.prod.qCom.valor     = '100.00'
    d1.prod.vUnCom.valor   = '10.0000'
    d1.prod.vProd.valor    = '1000.00'
    d1.prod.cEANTrib.valor = ''
    d1.prod.uTrib.valor    = d1.prod.uCom.valor
    d1.prod.qTrib.valor    = d1.prod.qCom.valor
    d1.prod.vUnTrib.valor  = d1.prod.vUnCom.valor
    d1.prod.vFrete.valor   = '0.00'
    d1.prod.vSeg.valor     = '0.00'
    d1.prod.vDesc.valor    = '0.00'
    d1.prod.vOutro.valor   = '0.00'
    #
    # Produto entra no total da NF-e
    #
    d1.prod.indTot.valor   = 1

    #
    # Impostos
    #
    d1.imposto.ICMS.CST.valor   = '00'
    d1.imposto.ICMS.modBC.valor = 3
    d1.imposto.ICMS.vBC.valor   = '1000.00'
    d1.imposto.ICMS.pICMS.valor = '18.00'
    d1.imposto.ICMS.vICMS.valor = '180.00'

    d1.imposto.IPI.CST.valor    = '50'
    d1.imposto.IPI.vBC.valor    = '1000.00'
    d1.imposto.IPI.pIPI.valor   = '10.00'
    d1.imposto.IPI.vIPI.valor   = '100.00'

    d1.imposto.PIS.CST.valor    = '01'
    d1.imposto.PIS.vBC.valor    = '1000.00'
    d1.imposto.PIS.pPIS.valor   = '0.65'
    d1.imposto.PIS.vPIS.valor   = '6.50'

    d1.imposto.COFINS.CST.valor    = '01'
    d1.imposto.COFINS.vBC.valor    = '1000.00'
    d1.imposto.COFINS.pCOFINS.valor   = '3.00'
    d1.imposto.COFINS.vCOFINS.valor   = '30.00'

    #
    # Os primeiros 188 caracteres desta string
    # são todos os caracteres válidos em tags da NF-e
    #
    d1.infAdProd.valor = '!"#$%&\'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_`abcdefghijklmnopqrstuvwxyz{|}~¡¢£¤¥¦§¨©ª«¬®¯°±²³´µ·¸¹º»¼½¾¿ÀÁÂÃÄÅÆÇÈÉÊËÌÍÎÏÐÑÒÓÔÕÖ×ØÙÚÛÜÝÞßàáâãäåæçèéêëìíîïðñòóôõö÷øùúûüýþÿ !"#$%&\'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_`abcdefghijklmnopqrstuvwxyz{|}~¡¢£¤¥¦§¨©ª«¬®¯°±²³´µ·¸¹º»¼½¾¿ÀÁÂÃÄÅÆÇÈÉÊËÌÍÎÏÐÑÒÓÔÕÖ×ØÙÚÛÜÝÞßàáâãäåæçèéêëìíîïðñòóôõö÷øùúûüýþÿ !"#$%&\'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_`abcdefghijklmnopqrstuvwxyz{|}~¡¢£¤¥¦§¨©ª«¬®¯°±²³´µ·¸¹º»¼½¾¿À'

    #
    # Inclui o detalhe na NF-e
    #
    n.infNFe.det.append(d1)

    #
    # Totais
    #
    n.infNFe.total.ICMSTot.vBC.valor     = '1000.00'
    n.infNFe.total.ICMSTot.vICMS.valor   = '180.00'
    n.infNFe.total.ICMSTot.vBCST.valor   = '0.00'
    n.infNFe.total.ICMSTot.vST.valor     = '0.00'
    n.infNFe.total.ICMSTot.vProd.valor   = '1000.00'
    n.infNFe.total.ICMSTot.vFrete.valor  = '0.00'
    n.infNFe.total.ICMSTot.vSeg.valor    = '0.00'
    n.infNFe.total.ICMSTot.vDesc.valor   = '0.00'
    n.infNFe.total.ICMSTot.vII.valor     = '0.00'
    n.infNFe.total.ICMSTot.vIPI.valor    = '100.00'
    n.infNFe.total.ICMSTot.vPIS.valor    = '6.50'
    n.infNFe.total.ICMSTot.vCOFINS.valor = '30.00'
    n.infNFe.total.ICMSTot.vOutro.valor  = '0.00'
    n.infNFe.total.ICMSTot.vNF.valor     = '1100.00'
    n.gera_nova_chave()

    #
    # O retorno de cada webservice é um objeto
    # com as seguintes propriedades
    #  .webservice - o webservice que foi consultado
    #  .envio - o objeto da classe XMLNFE enviado
    #  .envio.original - o texto do xml (envelope SOAP) enviado ao webservice
    #  .resposta - o objeto da classe XMLNFE retornado
    #  .resposta.version - version da HTTPResponse
    #  .resposta.status - status da HTTPResponse
    #  .resposta.reason - reason da HTTPResponse
    #  .resposta.msg - msg da HTTPResponse
    #  .resposta.original - o texto do xml (SOAP) recebido do webservice
    #
    for processo in p.processar_notas([n]):
        print(processo)
        print()
        print(processo.envio.xml)
        print()
        print(processo.envio.original)
        print()
        print(processo.resposta.xml)
        print()
        print(processo.resposta.original)
        print()
        print(processo.resposta.reason)

        #
        # A consulta dos recibos também retorna dois dicionários, cujas chaves
        # são as chaves das NF-es enviadas;
        #    . dic_protNFe - dicionário com os protocolos de cada NF-e
        #    . dic_procNFe - dicionário com os processos (NF-e + protocolo) de cada NF-e
        #    cada procNFe tem ainda uma propriedade:
        #        .danfe_pdf - conteúdo binário do DANFE em PDF
        #
        if processo.webservice == WS_NFE_CONSULTA_RECIBO:
            print()
            print(processo.resposta.dic_protNFe)
            print()
            print(processo.resposta.dic_procNFe)




# -*- coding: utf-8 -*-
#
# PySPED - Python libraries to deal with Brazil's SPED Project
#
# Copyright (C) 2010-2012
# Copyright (C) Aristides Caldeira <aristides.caldeira at tauga.com.br>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Library General Public License as
# published by the Free Software Foundation, either version 2.1 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# PySPED - Bibliotecas Python para o
#          SPED - Sistema Público de Escrituração Digital
#
# Copyright (C) 2010-2012
# Copyright (C) Aristides Caldeira <aristides.caldeira arroba tauga.com.br>
#
# Este programa é um software livre: você pode redistribuir e/ou modificar
# este programa sob os termos da licença GNU Library General Public License,
# publicada pela Free Software Foundation, em sua versão 2.1 ou, de acordo
# com sua opção, qualquer versão posterior.
#
# Este programa é distribuido na esperança de que venha a ser útil,
# porém SEM QUAISQUER GARANTIAS, nem mesmo a garantia implícita de
# COMERCIABILIDADE ou ADEQUAÇÃO A UMA FINALIDADE ESPECÍFICA. Veja a
# GNU Library General Public License para mais detalhes.
#
# Você deve ter recebido uma cópia da GNU Library General Public License
# juntamente com este programa. Caso esse não seja o caso, acesse:
# <http://www.gnu.org/licenses/>
#

from __future__ import division, print_function, unicode_literals

from pysped.xml_sped import (ABERTURA, NAMESPACE_CTE, Signature, TagCaracter,
                             TagDataHora, TagDecimal, TagInteiro, XMLNFe)
from pysped.cte.leiaute import ESQUEMA_ATUAL_VERSAO_104 as ESQUEMA_ATUAL
import os
from cte_104 import CTe


DIRNAME = os.path.dirname(__file__)


class ConsReciCTe(XMLNFe):
    def __init__(self):
        super(ConsReciCTe, self).__init__()
        self.versao  = TagDecimal(nome='consReciCTe', codigo='BP02', propriedade='versao', namespace=NAMESPACE_CTE, valor='1.04', raiz='/')
        self.tpAmb   = TagInteiro(nome='tpAmb'      , codigo='BP03', tamanho=[1,   1, 1]  , raiz='//consReciCTe')
        self.nRec    = TagCaracter(nome='nRec'      , codigo='BP04', tamanho=[1, 15, 1]   , raiz='//consReciCTe')
        self.caminho_esquema = os.path.join(DIRNAME, 'schema/', ESQUEMA_ATUAL + '/')
        self.arquivo_esquema = 'consReciCte_v1.04.xsd'

    def get_xml(self):
        xml = XMLNFe.get_xml(self)
        xml += ABERTURA
        xml += self.versao.xml
        xml += self.tpAmb.xml
        xml += self.nRec.xml
        xml += '</consReciCTe>'
        return xml

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.versao.xml = arquivo
            self.tpAmb.xml  = arquivo
            self.nRec.xml   = arquivo

        return self.xml

    xml = property(get_xml, set_xml)


class InfProt(XMLNFe):
    def __init__(self):
        super(InfProt, self).__init__()
        self.Id        = TagCaracter(nome='infProt' , codigo='PR04', propriedade='Id'  , raiz='/'        , obrigatorio=False, namespace=NAMESPACE_CTE)
        self.tpAmb     = TagInteiro(nome='tpAmb'    , codigo='PR05', tamanho=[1,   1, 1], raiz='//infProt', namespace=NAMESPACE_CTE)
        self.verAplic  = TagCaracter(nome='verAplic', codigo='PR06', tamanho=[1,  20]   , raiz='//infProt', namespace=NAMESPACE_CTE)
        self.chCTe     = TagCaracter(nome='chCTe'   , codigo='PR07', tamanho=[44, 44]   , raiz='//infProt', namespace=NAMESPACE_CTE)
        self.dhRecbto  = TagDataHora(nome='dhRecbto', codigo='PR08'                     , raiz='//infProt', namespace=NAMESPACE_CTE)
        self.nProt     = TagCaracter(nome='nProt'   , codigo='PR09', tamanho=[15, 15]   , raiz='//infProt', obrigatorio=False, namespace=NAMESPACE_CTE)
        self.digVal    = TagCaracter(nome='digVal'  , codigo='PR10', tamanho=[28, 28]   , raiz='//infProt', obrigatorio=False, namespace=NAMESPACE_CTE)
        self.cStat     = TagCaracter(nome='cStat'   , codigo='PR11' , tamanho=[1,   3]  , raiz='//infProt', namespace=NAMESPACE_CTE)
        self.xMotivo   = TagCaracter(nome='xMotivo' , codigo='PR12' , tamanho=[1, 255]  , raiz='//infProt', namespace=NAMESPACE_CTE)

    def get_xml(self):
        xml = XMLNFe.get_xml(self)

        if self.Id.valor:
            xml += self.Id.xml
        else:
            xml += '<infProt>'

        xml += self.tpAmb.xml
        xml += self.verAplic.xml
        xml += self.chCTe.xml
        xml += self.dhRecbto.xml
        xml += self.nProt.xml
        xml += self.digVal.xml
        xml += self.cStat.xml
        xml += self.xMotivo.xml
        xml += '</infProt>'
        return xml

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.Id.xml        = arquivo
            self.tpAmb.xml     = arquivo
            self.verAplic.xml  = arquivo
            self.chCTe.xml     = arquivo
            self.dhRecbto.xml  = arquivo
            self.nProt.xml     = arquivo
            self.digVal.xml    = arquivo
            self.cStat.xml     = arquivo
            self.xMotivo.xml   = arquivo

    xml = property(get_xml, set_xml)


class ProtCTe(XMLNFe):
    def __init__(self):
        super(ProtCTe, self).__init__()
        self.versao  = TagDecimal(nome='protCTe', codigo='PR02' , propriedade='versao', namespace=NAMESPACE_CTE, valor='1.04', raiz='/')
        self.infProt = InfProt()
        self.Signature = Signature()

    def get_xml(self):
        xml = XMLNFe.get_xml(self)
        xml += self.versao.xml
        xml += self.infProt.xml

        if len(self.Signature.URI) and (self.Signature.URI.strip() != '#'):
            xml += self.Signature.xml

        xml += '</protCTe>'
        return xml

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.versao.xml  = arquivo
            #
            # o grupo infProt é usado também no webservice de consulta da situação de uma NF-e
            # por isso, a raiz dele não pode ser assumida como sendo sempre o grupo
            # protCTe
            #
            print('aqui:', self._le_noh('//protCTe/infProt', ns=NAMESPACE_CTE))
            self.infProt.xml = self._le_noh('//protCTe/infProt', ns=NAMESPACE_CTE)
            self.Signature.xml = self._le_noh('//protCTe/sig:Signature')

    xml = property(get_xml, set_xml)

    def protocolo_formatado(self):
        if not self.infProt.nProt.valor:
            return ''

        formatado = self.infProt.nProt.valor
        formatado += ' - '
        formatado += self.infProt.dhRecbto.formato_danfe()
        return formatado


class RetConsReciCTe(XMLNFe):
    def __init__(self):
        super(RetConsReciCTe, self).__init__()
        self.versao   = TagDecimal(nome='retConsReciCTe', codigo='BR02' , propriedade='versao', namespace=NAMESPACE_CTE, valor='1.04', raiz='/')
        self.tpAmb    = TagInteiro(nome='tpAmb'         , codigo='BR03' , tamanho=[1,   1, 1], raiz='//retConsReciCTe')
        self.verAplic = TagCaracter(nome='verAplic'     , codigo='BR04' , tamanho=[1,  20]   , raiz='//retConsReciCTe')
        self.nRec     = TagCaracter(nome='nRec'         , codigo='BR04a', tamanho=[1, 15, 1] , raiz='//retConsReciCTe')
        self.cStat    = TagCaracter(nome='cStat'        , codigo='BR05' , tamanho=[1,   3]   , raiz='//retConsReciCTe')
        self.xMotivo  = TagCaracter(nome='xMotivo'      , codigo='BR06' , tamanho=[1, 255]   , raiz='//retConsReciCTe')
        self.cUF      = TagCaracter(nome='cUF'          , codigo='BR06a', tamanho=[2,   2, 2], raiz='//retConsReciCTe')
        self.protCTe  = []

        #
        # Dicionário dos protocolos, com a chave sendo a chave de NF-e
        #
        self.dic_protCTe = {}
        #
        # Dicionário dos processos (NF-e + protocolo), com a chave sendo a chave da NF-e
        #
        self.dic_procCTe = {}

        self.caminho_esquema = os.path.join(DIRNAME, 'schema/', ESQUEMA_ATUAL + '/')
        self.arquivo_esquema = 'retConsReciCte_v1.04.xsd'


    def get_xml(self):
        xml = XMLNFe.get_xml(self)
        xml += self.versao.xml
        xml += self.tpAmb.xml
        xml += self.verAplic.xml
        xml += self.nRec.xml
        xml += self.cStat.xml
        xml += self.xMotivo.xml
        xml += self.cUF.xml

        for pn in self.protCTe:
            xml += pn.xml

        xml += '</retConsReciCTe>'
        return xml

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.versao.xml   = arquivo
            self.tpAmb.xml    = arquivo
            self.verAplic.xml = arquivo
            self.nRec.xml     = arquivo
            self.cStat.xml    = arquivo
            self.xMotivo.xml  = arquivo
            self.cUF.xml      = arquivo
            self.protCTe      = self.le_grupo('//retConsReciCTe/protCTe', ProtCTe)

            #
            # Monta o dicionário dos protocolos
            #
            for pn in self.protCTe:
                self.dic_protCTe[pn.infProt.chCTe.valor] = pn

    xml = property(get_xml, set_xml)


class ProcCTe(XMLNFe):
    def __init__(self):
        super(ProcCTe, self).__init__()
        self.versao  = TagDecimal(nome='cteProc', propriedade='versao', namespace=NAMESPACE_CTE, valor='1.04', raiz='/')
        self.CTe     = CTe()
        self.protCTe = ProtCTe()
        self.caminho_esquema = os.path.join(DIRNAME, 'schema/', ESQUEMA_ATUAL + '/')
        self.arquivo_esquema = 'procCTe_v1.04.xsd'

    def get_xml(self):
        xml = XMLNFe.get_xml(self)
        xml += ABERTURA
        xml += self.versao.xml
        xml += self.CTe.xml.replace(ABERTURA, '')
        xml += self.protCTe.xml.replace(ABERTURA, '')
        xml += '</cteProc>'
        return xml

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.CTe.xml     = arquivo
            self.protCTe.xml = self._le_noh('//cteProc/protCTe', ns=NAMESPACE_CTE)

    xml = property(get_xml, set_xml)



# -*- coding: utf-8 -*-
#
# PySPED - Python libraries to deal with Brazil's SPED Project
#
# Copyright (C) 2010-2012
# Copyright (C) Aristides Caldeira <aristides.caldeira at tauga.com.br>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# PySPED - Bibliotecas Python para o
#          SPED - Sistema Público de Escrituração Digital
#
# Copyright (C) 2010-2012
# Copyright (C) Aristides Caldeira <aristides.caldeira arroba tauga.com.br>
#
# Este programa é um software livre: você pode redistribuir e/ou modificar
# este programa sob os termos da licença GNU Affero General Public License,
# publicada pela Free Software Foundation, em sua versão 3 ou, de acordo
# com sua opção, qualquer versão posterior.
#
# Este programa é distribuido na esperança de que venha a ser útil,
# porém SEM QUAISQUER GARANTIAS, nem mesmo a garantia implícita de
# COMERCIABILIDADE ou ADEQUAÇÃO A UMA FINALIDADE ESPECÍFICA. Veja a
# GNU Affero General Public License para mais detalhes.
#
# Você deve ter recebido uma cópia da GNU Affero General Public License
# juntamente com este programa. Caso esse não seja o caso, acesse:
# <http://www.gnu.org/licenses/>
#

from __future__ import division, print_function, unicode_literals

from pysped.xml_sped import *
from pysped.nfe.leiaute import ESQUEMA_ATUAL_VERSAO_3 as ESQUEMA_ATUAL
from pysped.nfe.leiaute import nfe_310
from pysped.nfe.webservices_flags import CODIGO_UF
from pysped.nfe.webservices_nfce_3 import ESTADO_QRCODE, ESTADO_CONSULTA_NFCE
import os
import binascii
import hashlib
import qrcode
from StringIO import StringIO


DIRNAME = os.path.dirname(__file__)

class CSC(object):
    def __init__(self):
        self.id = '1'
        self.codigo = ''


class InfNFe(nfe_310.InfNFe):
    def __init__(self):
        super(InfNFe, self).__init__()
        self.ide.mod.valor = '65'  #  NFC-e
        self.ide.tpImp.valor = '4'  #  DANFE NFC-e em papel
        self.ide.indPres.valor = '1'  #  Operação presencial
        self.ide.indFinal.valor = '1'  #  Consumidor final
        self.transp.modFrete.valor = 9  #  Sem frete
        self.dest.modelo = '65'
        self.emit.csc = CSC()


class InfNFeSupl(XMLNFe):
    def __init__(self):
        super(InfNFeSupl, self).__init__()
        self.qrCode = TagCaracter(nome='qrCode', codigo='', tamanho=[1,  600], raiz='//NFe/infNFeSupl', cdata=True)

    def get_xml(self):
        xml = XMLNFe.get_xml(self)
        xml += '<infNFeSupl>'
        xml += self.qrCode.xml
        xml += '</infNFeSupl>'
        return xml

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.qrCode.xml = arquivo

    xml = property(get_xml, set_xml)


class NFCe(nfe_310.NFe):
    def __init__(self):
        super(NFCe, self).__init__()
        self.infNFe = InfNFe()
        self.qrcode = ''
        self.infNFeSupl = InfNFeSupl()
        self.Signature = Signature()
        self.caminho_esquema = os.path.join(DIRNAME, 'schema/', ESQUEMA_ATUAL + '/')
        self.arquivo_esquema = 'nfe_v3.10.xsd'

    def get_xml(self):
        xml = XMLNFe.get_xml(self)
        xml += ABERTURA
        xml += '<NFe xmlns="http://www.portalfiscal.inf.br/nfe">'
        xml += self.infNFe.xml
        xml += self.infNFeSupl.xml

        #
        # Define a URI a ser assinada
        #
        self.Signature.URI = '#' + self.infNFe.Id.valor

        xml += self.Signature.xml
        xml += '</NFe>'
        return xml

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.infNFe.xml     = arquivo
            self.infNFeSupl.xml = arquivo
            self.Signature.xml  = self._le_noh('//NFe/sig:Signature')

    xml = property(get_xml, set_xml)

    def monta_chave(self):
        chave = unicode(self.infNFe.ide.cUF.valor).strip().rjust(2, '0')
        chave += unicode(self.infNFe.ide.dhEmi.valor.strftime('%y%m')).strip().rjust(4, '0')
        chave += unicode(self.infNFe.emit.CNPJ.valor).strip().rjust(14, '0')
        chave += '65'
        chave += unicode(self.infNFe.ide.serie.valor).strip().rjust(3, '0')
        chave += unicode(self.infNFe.ide.nNF.valor).strip().rjust(9, '0')

        #
        # Inclui agora o tipo da emissão
        #
        chave += unicode(self.infNFe.ide.tpEmis.valor).strip().rjust(1, '0')

        chave += unicode(self.infNFe.ide.cNF.valor).strip().rjust(8, '0')
        chave += unicode(self.infNFe.ide.cDV.valor).strip().rjust(1, '0')
        self.chave = chave

    def monta_qrcode(self):
        self.monta_chave()

        qrcode = 'chNFe=' + self.chave
        qrcode += '&nVersao=100'
        qrcode += '&tpAmb=' + self.infNFe.ide.tpAmb._valor_string

        if self.infNFe.dest.CNPJ.valor:
            qrcode += '&cDest=' + self.infNFe.dest.CNPJ._valor_string

        elif self.infNFe.dest.CPF.valor:
            qrcode += '&cDest=' + self.infNFe.dest.CPF._valor_string

        elif self.infNFe.dest.idEstrangeiro.valor:
            qrcode += '&cDest=' + self.infNFe.dest.idEstrangeiro._valor_string

        #
        # SP, PR e MS
        #
        if self.infNFe.ide.cUF.valor not in (35, 41, 50):
            qrcode += '&dhEmi=' + binascii.hexlify(self.infNFe.ide.dhEmi._valor_string)
        else:
            qrcode += '&dhEmi=' + binascii.hexlify(self.infNFe.ide.dhEmi._valor_string).upper()

        qrcode += '&vNF=' + self.infNFe.total.ICMSTot.vNF._valor_string
        qrcode += '&vICMS=' + self.infNFe.total.ICMSTot.vICMS._valor_string

        if self.infNFe.ide.cUF.valor not in (35, 41, 50):
            qrcode += '&digVal=' + binascii.hexlify(self.Signature.DigestValue)
        else:
            qrcode += '&digVal=' + binascii.hexlify(self.Signature.DigestValue).upper()

        qrcode += '&cIdToken=' + str(self.infNFe.emit.csc.id).zfill(6)

        pre_qrcode = qrcode + self.infNFe.emit.csc.codigo.ljust(36).upper()

        qrcode += '&cHashQRCode=' + hashlib.sha1(pre_qrcode).hexdigest().upper()

        self.qrcode = qrcode

        qrcode = ESTADO_QRCODE[CODIGO_UF[self.infNFe.ide.cUF.valor]][self.infNFe.ide.tpAmb.valor] + '?' + qrcode

        self.infNFeSupl.qrCode.valor = qrcode

    @property
    def qrcode_imagem(self):
        #
        # box_size=2 dá um tamanho de 4 x 4 cm
        #
        codigo = qrcode.QRCode(box_size=2)
        codigo.add_data(self.infNFeSupl.qrCode.valor)

        arq = StringIO()
        codigo.make_image().save(arq)
        arq.pos = 0
        imagem = arq.read().encode('base64')
        arq.close()

        return imagem

    @property
    def numero_formatado(self):
        num = unicode(self.infNFe.ide.nNF.valor).zfill(9)
        num_formatado = '.'.join((num[0:3], num[3:6], num[6:9]))
        return 'nº ' + num_formatado

    @property
    def serie_formatada(self):
        return 'Série ' + unicode(self.infNFe.ide.serie.valor).zfill(3)

    @property
    def url_consulta(self):
        return ESTADO_CONSULTA_NFCE[CODIGO_UF[self.infNFe.ide.cUF.valor]][self.infNFe.ide.tpAmb.valor]

    @property
    def cnpj_destinatario_formatado(self):
        if self.infNFe.dest.CPF.valor and len(self.infNFe.dest.CPF.valor):
            return 'CNPJ ' + self._formata_cpf(unicode(self.infNFe.dest.CPF.valor))
        elif self.infNFe.dest.CNPJ.valor and len(self.infNFe.dest.CNPJ.valor):
            return 'CPF ' + self._formata_cnpj(unicode(self.infNFe.dest.CNPJ.valor))
        elif self.infNFe.dest.idEstrangeiro.valor and len(self.infNFe.dest.idEstrangeiro.valor):
            return 'Id. estrangeiro ' + self.infNFe.dest.idEstrangeiro.valor
        else:
            return ''

    @property
    def quantidade_itens(self):
        return str(int(len(self.infNFe.det)))



# -*- coding: utf-8 -*-
#
# PySPED - Python libraries to deal with Brazil's SPED Project
#
# Copyright (C) 2010-2012
# Copyright (C) Aristides Caldeira <aristides.caldeira at tauga.com.br>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Library General Public License as
# published by the Free Software Foundation, either version 2.1 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# PySPED - Bibliotecas Python para o
#          SPED - Sistema Público de Escrituração Digital
#
# Copyright (C) 2010-2012
# Copyright (C) Aristides Caldeira <aristides.caldeira arroba tauga.com.br>
#
# Este programa é um software livre: você pode redistribuir e/ou modificar
# este programa sob os termos da licença GNU Library General Public License,
# publicada pela Free Software Foundation, em sua versão 2.1 ou, de acordo
# com sua opção, qualquer versão posterior.
#
# Este programa é distribuido na esperança de que venha a ser útil,
# porém SEM QUAISQUER GARANTIAS, nem mesmo a garantia implícita de
# COMERCIABILIDADE ou ADEQUAÇÃO A UMA FINALIDADE ESPECÍFICA. Veja a
# GNU Library General Public License para mais detalhes.
#
# Você deve ter recebido uma cópia da GNU Library General Public License
# juntamente com este programa. Caso esse não seja o caso, acesse:
# <http://www.gnu.org/licenses/>
#

from __future__ import division, print_function, unicode_literals

from lxml import etree
from datetime import datetime, date, time
from decimal import Decimal
import locale
import unicodedata
import re
import pytz
from time import strftime


NAMESPACE_NFE = 'http://www.portalfiscal.inf.br/nfe'
NAMESPACE_CTE = 'http://www.portalfiscal.inf.br/cte'
NAMESPACE_SIG = 'http://www.w3.org/2000/09/xmldsig#'
NAMESPACE_NFSE = 'http://localhost:8080/WsNFe2/lote'
ABERTURA = '<?xml version="1.0" encoding="utf-8"?>'

locale.setlocale(locale.LC_ALL, b'pt_BR.UTF-8')
locale.setlocale(locale.LC_COLLATE, b'pt_BR.UTF-8')


class NohXML(object):
    def __init__(self, *args, **kwargs):
        self._xml = None
        self.alertas = []

    def _le_xml(self, arquivo):
        if arquivo is None:
            return False

        if not isinstance(arquivo, basestring):
            arquivo = etree.tounicode(arquivo)
            #self._xml = arquivo
            #return True

        #elif arquivo is not None:
        if arquivo is not None:
            if isinstance(arquivo, basestring):
                if isinstance(arquivo, str):
                    arquivo = unicode(arquivo.encode('utf-8'))

                if '<' in arquivo:
                    self._xml = etree.fromstring(tira_abertura(arquivo).encode('utf-8'))
                else:
                    arq = open(arquivo)
                    txt = b''.join(arq.readlines())
                    txt = unicode(txt.decode('utf-8'))
                    txt = tira_abertura(txt)
                    arq.close()
                    self._xml = etree.fromstring(txt)
            else:
                self._xml = etree.parse(arquivo)
            return True

        return False

    def _preenche_namespace(self, tag, sigla_ns):
        if sigla_ns != '':
            sigla_sig = sigla_ns + ':sig'
            sigla_ns = '/' + sigla_ns + ':'
            tag = sigla_ns.join(tag.split('/')).replace(sigla_ns + sigla_ns, '/' + sigla_ns).replace(sigla_sig, 'sig')

        return tag

    def _le_nohs(self, tag, ns=None, sigla_ns='nfe'):
        #
        # Tenta ler a tag sem os namespaces
        # Necessário para ler corretamente as tags de grupo reenraizadas
        #
        try:
            nohs = self._xml.xpath(tag)
            if len(nohs) >= 1:
                return nohs
        except:
            pass

        #
        # Não deu certo, tem que botar mesmo os namespaces
        #
        namespaces = {'nfe': NAMESPACE_NFE, 'sig': NAMESPACE_SIG, 'nfse': NAMESPACE_NFSE, 'cte': NAMESPACE_CTE}

        if ns is not None:
            namespaces['res'] = ns

        if '//NFe' in tag or ns == NAMESPACE_NFE:
            sigla_ns = 'nfe'
        elif '//CTe' in tag or ns == NAMESPACE_CTE:
            sigla_ns = 'cte'

        if not tag.startswith('//*/res'):
            tag = self._preenche_namespace(tag, sigla_ns)

        nohs = self._xml.xpath(tag, namespaces=namespaces)

        if len(nohs) >= 1:
            return nohs
        else:
            return None

    def _le_noh(self, tag, ns=None, ocorrencia=1):
        nohs = self._le_nohs(tag, ns)

        if (nohs is not None) and (len(nohs) >= ocorrencia):
            return nohs[ocorrencia-1]
        else:
            return None

    def _le_tag(self, tag, propriedade=None, ns=None, ocorrencia=1):
        noh = self._le_noh(tag,  ns, ocorrencia)

        if noh is None:
            valor = ''
        else:
            if propriedade is None:
                valor = noh.text
            elif (noh.attrib is not None) and (len(noh.attrib) > 0):
                valor = noh.attrib[propriedade]
            else:
                valor = ''

        return valor


class ErroObrigatorio(Exception):
    def __init__(self, codigo, nome, propriedade):
        if propriedade:
            self.value = 'No campo código ' + codigo + ', "' + nome + '", a propriedade "' + propriedade + '" é de envio obrigatório, mas não foi preenchida.'
        else:
            self.value = 'O campo código ' + codigo + ', "' + nome + '" é de envio obrigatório, mas não foi preenchido.'

    def __str__(self):
        return repr(self.value)

    def __unicode__(self):
        return unicode(self.value)


class TamanhoInvalido(Exception):
    def __init__(self, codigo, nome, valor, tam_min=None, tam_max=None, dec_min=None, dec_max=None):
        if tam_min:
           self.value = 'O campo código ' + codigo + ', "' + nome + '", deve ter o tamanho mínimo de ' + unicode(tam_min) + ', mas o tamanho enviado foi ' + unicode(len(unicode(valor))) + ': ' + unicode(valor)
        elif tam_max:
           self.value = 'O campo código ' + codigo + ', "' + nome + '", deve ter o tamanho máximo de ' + unicode(tam_max) + ', mas o tamanho enviado foi ' + unicode(len(unicode(valor))) + ': ' + unicode(valor)
        elif dec_min:
           self.value = 'O campo código ' + codigo + ', "' + nome + '", deve ter o mínimo de ' + unicode(dec_min) + ' casas decimais, mas o enviado foi ' + unicode(len(unicode(valor))) + ': ' + unicode(valor)
        elif dec_max:
           self.value = 'O campo código ' + codigo + ', "' + nome + '", deve ter o máximo de ' + unicode(dec_max) + ' casas decimais, mas o enviado foi ' + unicode(len(unicode(valor))) + ': ' + unicode(valor)

    def __str__(self):
        return repr(self.value)

    def __unicode__(self):
        return unicode(self.value)


class ErroCaracterInvalido(Exception):
    def __init__(self, codigo, nome, propriedade, valor, caracter):
        if propriedade:
            self.value = 'No campo código ' + codigo + ', "' + nome + '", a propriedade "' + propriedade + '" possui um caracter inválido: "' + caracter + '".'
        else:
            self.value = 'O campo código ' + codigo + ', "' + nome + '" possui um caracter inválido: "' + caracter + '".'

    def __str__(self):
        return repr(self.value)

    def __unicode__(self):
        return unicode(self.value)


class TagCaracter(NohXML):
    def __init__(self, *args, **kwargs):
        super(TagCaracter, self).__init__(*args, **kwargs)
        self.codigo = ''
        self.nome = ''
        self._valor_string = ''
        self.obrigatorio = True
        self.tamanho = [None, None, None]
        self.propriedade = None
        self.namespace = None
        self.namespace_obrigatorio = True
        self.alertas = []
        self.raiz = None
        self.cdata = False

        # Codigo para dinamizar a criacao de instancias de entidade,
        # aplicando os valores dos atributos na instanciacao
        for k, v in kwargs.items():
            setattr(self, k, v)

        if kwargs.has_key('valor'):
            self.valor = kwargs['valor']

    def _testa_obrigatorio(self, valor):
        if self.obrigatorio and (not valor):
            return ErroObrigatorio(self.codigo, self.nome, self.propriedade)
            #raise ErroObrigatorio(self.codigo, self.nome, self.propriedade)

    def _testa_tamanho_minimo(self, valor):
        if self.tamanho[0] and (len(unicode(valor)) < self.tamanho[0]):
            return TamanhoInvalido(self.codigo, self.nome, valor, tam_min=self.tamanho[0])
            #raise TamanhoInvalido(self.codigo, self.nome, valor, tam_min=self.tamanho[0])

    def _testa_tamanho_maximo(self, valor):
        if self.tamanho[1] and (len(unicode(valor)) > self.tamanho[1]):
            return TamanhoInvalido(self.codigo, self.nome, valor, tam_max=self.tamanho[1])
            #raise TamanhoInvalido(self.codigo, self.nome, valor, tam_max=self.tamanho[1])

    def _valida(self, valor):
        self.alertas = []

        v = valor
        if self.cdata:
            v = valor.replace('<![CDATA[', '')
            v = v.replace(']]>', '')

        if self._testa_obrigatorio(v):
            self.alertas.append(self._testa_obrigatorio(v))

        if self._testa_tamanho_minimo(v):
            self.alertas.append(self._testa_tamanho_minimo(v))

        if self._testa_tamanho_maximo(v):
            self.alertas.append(self._testa_tamanho_maximo(v))

        return self.alertas == []

    def set_valor(self, novo_valor):
        if novo_valor is not None:
            novo_valor = unicode(novo_valor)
            #
            # Remover caratceres inválidos
            #
            for c in novo_valor:
                if c > 'ÿ':
                    raise ErroCaracterInvalido(self.codigo, self.nome, self.propriedade, novo_valor, c)

            #
            # É obrigatório remover os espaços no início e no final do valor
            #
            novo_valor = novo_valor.strip()

        if self._valida(novo_valor):
            if self.cdata:
                self._valor_string = unicode(novo_valor)
            else:
                self._valor_string = unicode(tirar_acentos(novo_valor))
        else:
            self._valor_string = ''

    def get_valor(self):
        return unicode(por_acentos(self._valor_string))

    valor = property(get_valor, set_valor)

    def __unicode__(self):
        if (not self.obrigatorio) and (not self.valor):
            texto = ''
        else:
            texto = '<%s' % self.nome

            if self.namespace and self.namespace_obrigatorio:
                texto += ' xmlns="%s"' % self.namespace

            if self.propriedade:
                texto += ' %s="%s">' % (self.propriedade, self._valor_string)
            elif self.valor or (len(self.tamanho) == 3 and self.tamanho[2]):
                if self.cdata:
                    texto += '><![CDATA[%s]]></%s>' % (self._valor_string, self.nome)
                else:
                    texto += '>%s</%s>' % (self._valor_string, self.nome)
            else:
                texto += ' />'

        return texto

    def __repr__(self):
        return self.__unicode__()

    def get_xml(self):
        return self.__unicode__()

    def set_xml(self, arquivo, ocorrencia=1):
        if self._le_xml(arquivo):
            self.valor = self._le_tag(self.raiz + '/' + self.nome, propriedade=self.propriedade, ns=self.namespace, ocorrencia=ocorrencia)

    xml = property(get_xml, set_xml)

    def get_text(self):
        if self.propriedade:
            return '%s_%s=%s' % (self.nome, self.propriedade, self._valor_string)
        else:
            return '%s=%s' % (self.nome, self._valor_string)

    text = property(get_text)

    def get_txt(self):
        if self.obrigatorio:
            return self._valor_string

        if self.valor:
            return self._valor_string

        return ''

    txt = property(get_txt)


class TagBoolean(TagCaracter):
    def __init__(self, **kwargs):
        super(TagBoolean, self).__init__(**kwargs)
        self._valor_boolean = None
        # Codigo para dinamizar a criacao de instancias de entidade,
        # aplicando os valores dos atributos na instanciacao
        for k, v in kwargs.items():
            setattr(self, k, v)

        if kwargs.has_key('valor'):
            self.valor = kwargs['valor']


    def _testa_obrigatorio(self, valor):
        # No caso da tag booleana, False deve ser tratado como preenchido
        if self.obrigatorio and (valor is None):
            return ErroObrigatorio(self.codigo, self.nome, self.propriedade)

    def _valida(self, valor):
        self.alertas = []

        if self._testa_obrigatorio(valor):
            self.alertas.append(self._testa_obrigatorio(valor))

        return self.alertas == []

    def set_valor(self, novo_valor):
        if isinstance(novo_valor, basestring):
            if novo_valor.lower() == 'true':
                novo_valor = True
            elif novo_valor.lower() == 'false':
                novo_valor = False
            else:
                novo_valor = None

        if isinstance(novo_valor, bool) and self._valida(novo_valor):
            self._valor_boolean = novo_valor

            if novo_valor == None:
                self._valor_string = ''
            elif novo_valor:
                self._valor_string = 'true'
            else:
                self._valor_string = 'false'
        else:
            self._valor_boolean = None
            self._valor_string = ''

    def get_valor(self):
        return self._valor_boolean

    valor = property(get_valor, set_valor)

    def __unicode__(self):
        if (not self.obrigatorio) and (self.valor == None):
            texto = ''
        else:
            texto = '<%s' % self.nome

            if self.namespace:
                texto += ' xmlns="%s"' % self.namespace

            if self.propriedade:
                texto += ' %s="%s">' % (self.propriedade, self._valor_string)
            elif not self.valor == None:
                texto += '>%s</%s>' % (self._valor_string, self.nome)
            else:
                texto += ' />'

        return texto


class TagData(TagCaracter):
    def __init__(self, **kwargs):
        super(TagData, self).__init__(**kwargs)
        self._valor_data = None
        # Codigo para dinamizar a criacao de instancias de entidade,
        # aplicando os valores dos atributos na instanciacao
        for k, v in kwargs.items():
            setattr(self, k, v)

        if kwargs.has_key('valor'):
            self.valor = kwargs['valor']

    def _valida(self, valor):
        self.alertas = []

        if self._testa_obrigatorio(valor):
            self.alertas.append(self._testa_obrigatorio(valor))

        return self.alertas == []

    def set_valor(self, novo_valor):
        if isinstance(novo_valor, basestring):
            if novo_valor:
                novo_valor = datetime.strptime(novo_valor[:10], '%Y-%m-%d')
            else:
                novo_valor = None

        if isinstance(novo_valor, (datetime, date,)) and self._valida(novo_valor):
            self._valor_data = novo_valor
            # Cuidado!!!
            # Aqui não dá pra usar a função strftime pois em alguns
            # casos a data retornada é 01/01/0001 00:00:00
            # e a função strftime só aceita data com anos a partir de 1900
            self._valor_string = '%04d-%02d-%02d' % (self._valor_data.year, self._valor_data.month, self._valor_data.day)
        else:
            self._valor_data = None
            self._valor_string = ''

    def get_valor(self):
        return self._valor_data

    valor = property(get_valor, set_valor)

    @property
    def formato_danfe(self):
        if self._valor_data is None:
            return ''
        else:
            return self._valor_data.strftime('%d/%m/%Y')

class TagHora(TagData):
    def set_valor(self, novo_valor):
        if isinstance(novo_valor, basestring):
            if novo_valor:
                novo_valor = datetime.strptime(novo_valor[:8], '%H:%M:%S')
            else:
                novo_valor = None

        if isinstance(novo_valor, (datetime, time,)) and self._valida(novo_valor):
            self._valor_data = novo_valor
            # Cuidado!!!
            # Aqui não dá pra usar a função strftime pois em alguns
            # casos a data retornada é 01/01/0001 00:00:00
            # e a função strftime só aceita data com anos a partir de 1900
            self._valor_string = '%02d:%02d:%02d' % (self._valor_data.hour, self._valor_data.minute, self._valor_data.second)
        else:
            self._valor_data = None
            self._valor_string = ''

    def get_valor(self):
        return self._valor_data

    valor = property(get_valor, set_valor)

    @property
    def formato_danfe(self):
        if self._valor_data is None:
            return ''
        else:
            return self._valor_data.strftime('%H:%M:%S')


class TagDataHora(TagData):
    def set_valor(self, novo_valor):
        if isinstance(novo_valor, basestring):
            if novo_valor:
                #
                # Força a ignorar os microssegundos enviados pelo webservice
                # de distribuição de DF-e
                #
                if '.' in novo_valor:
                    novo_valor = novo_valor.split('.')[0]

                novo_valor = datetime.strptime(novo_valor, '%Y-%m-%dT%H:%M:%S')
            else:
                novo_valor = None

        if isinstance(novo_valor, datetime) and self._valida(novo_valor):
            self._valor_data = novo_valor
            self._valor_data = self._valor_data.replace(microsecond=0)
            # Cuidado!!!
            # Aqui não dá pra usar a função strftime pois em alguns
            # casos a data retornada é 01/01/0001 00:00:00
            # e a função strftime só aceita data com anos a partir de 1900
            self._valor_string = '%04d-%02d-%02dT%02d:%02d:%02d' % (self._valor_data.year, self._valor_data.month, self._valor_data.day,
                self._valor_data.hour, self._valor_data.minute, self._valor_data.second)
        else:
            self._valor_data = None
            self._valor_string = ''

    def get_valor(self):
        return self._valor_data

    valor = property(get_valor, set_valor)

    @property
    def formato_danfe(self):
        if self._valor_data is None:
            return ''
        else:
            return self._valor_data.strftime('%d/%m/%Y %H:%M:%S')


def fuso_horario_sistema():
    diferenca = int(strftime('%z')) // 100

    if diferenca < 0:
        return pytz.timezone('Etc/GMT+' + str(diferenca * -1))

    if diferenca > 0:
        return pytz.timezone('Etc/GMT-' + str(diferenca))

    return pytz.UTC


class TagDataHoraUTC(TagData):
    def __init__(self, **kwargs):
        super(TagDataHoraUTC, self).__init__(**kwargs)
        #
        # Expressão de validação do formato (vinda do arquivo leiauteSRE_V1.00.xsd
        # Alterada para tornar a informação do fuso horário opcional
        #
        self._validacao = re.compile(r'(((20(([02468][048])|([13579][26]))-02-29))|(20[0-9][0-9])-((((0[1-9])|(1[0-2]))-((0[1-9])|(1\d)|(2[0-8])))|((((0[13578])|(1[02]))-31)|(((0[1,3-9])|(1[0-2]))-(29|30)))))T(20|21|22|23|[0-1]\d):[0-5]\d:[0-5]\d(-0[1-4]:00)?')
        self._valida_fuso = re.compile(r'.*[-+]0[0-9]:00$')
        self._brasilia = pytz.timezone('America/Sao_Paulo')
        self.fuso_horario = 'America/Sao_Paulo'

    def set_valor(self, novo_valor):
        if isinstance(novo_valor, basestring):
            if self._validacao.match(novo_valor):
                if self._valida_fuso.match(novo_valor):
                    #
                    # Extrai e determina qual o fuso horário informado
                    #
                    self.fuso_horario = novo_valor[19:]
                    novo_valor = novo_valor[:19]

                #
                # Converte a data sem fuso horário para o fuso horário atual
                # Isso é necessário pois a função strptime ignora a informação
                # do fuso horário na string de entrada
                #
                novo_valor = self.fuso_horario.localize(datetime.strptime(novo_valor, '%Y-%m-%dT%H:%M:%S'))
            else:
                novo_valor = None

        if isinstance(novo_valor, datetime) and self._valida(novo_valor):

            if not novo_valor.tzinfo:
                novo_valor = fuso_horario_sistema().localize(novo_valor)
                novo_valor = pytz.UTC.normalize(novo_valor)
                novo_valor = self._brasilia.normalize(novo_valor)

            self._valor_data = novo_valor
            self._valor_data = self._valor_data.replace(microsecond=0)
            try:
                self._valor_data = self.fuso_horario.localize(self._valor_data)
            except:
                pass
            # Cuidado!!!
            # Aqui não dá pra usar a função strftime pois em alguns
            # casos a data retornada é 01/01/0001 00:00:00
            # e a função strftime só aceita data com anos a partir de 1900
            #self._valor_string = '%04d-%02d-%02dT%02d:%02d:%02d' % (self._valor_data.year, self._valor_data.month, self._valor_data.day,
            #    self._valor_data.hour, self._valor_data.minute, self._valor_data.second)

            self._valor_string = self._valor_data.isoformat()
        else:
            self._valor_data = None
            self._valor_string = ''

    def get_valor(self):
        return self._valor_data

    valor = property(get_valor, set_valor)

    def set_fuso_horaro(self, novo_valor):
        if novo_valor in pytz.country_timezones['br']:
            self._fuso_horario = pytz.timezone(novo_valor)

        #
        # Nos valores abaixo, não entendi ainda até agora, mas para o resultado
        # correto é preciso usar GMT+ (mais), não (menos) como seria de se
        # esperar...
        #
        elif novo_valor == '-04:00' or novo_valor == '-0400':
            self._fuso_horario = pytz.timezone('Etc/GMT+4')
        elif novo_valor == '-03:00' or novo_valor == '-0300':
            self._fuso_horario = pytz.timezone('Etc/GMT+3')
        elif novo_valor == '-02:00' or novo_valor == '-0200':
            self._fuso_horario = pytz.timezone('Etc/GMT+2')
        elif novo_valor == '-01:00' or novo_valor == '-0100':
            self._fuso_horario = pytz.timezone('Etc/GMT+1')

    def get_fuso_horario(self):
        return self._fuso_horario

    fuso_horario = property(get_fuso_horario, set_fuso_horaro)

    @property
    def formato_danfe(self):
        if self._valor_data is None:
            return ''
        else:
            valor = self._brasilia.normalize(self._valor_data).strftime('%d/%m/%Y %H:%M:%S %Z (%z)')
            #
            # Troca as siglas:
            # BRT - Brasília Time -> HOB - Horário Oficial de Brasília
            # BRST - Brasília Summer Time -> HVOB - Horário de Verão Oficial de Brasília
            # AMT - Amazon Time -> HOA - Horário Oficial da Amazônia
            # AMST - Amazon Summer Time -> HVOA - Horário de Verão Oficial da Amazônia
            # FNT - Fernando de Noronha Time -> HOFN - Horário Oficial de Fernando de Noronha
            #
            valor = valor.replace('(-0100)', '(-01:00)')
            valor = valor.replace('(-0200)', '(-02:00)')
            valor = valor.replace('(-0300)', '(-03:00)')
            valor = valor.replace('(-0400)', '(-04:00)')
            valor = valor.replace('BRT', 'HOB')
            valor = valor.replace('BRST', 'HVOB')
            valor = valor.replace('AMT', 'HOA')
            valor = valor.replace('AMST', 'HVOA')
            valor = valor.replace('FNT', 'HOFN')
            return valor


class TagInteiro(TagCaracter):
    def __init__(self, **kwargs):
        super(TagInteiro, self).__init__(**kwargs)
        self._valor_inteiro = 0
        self._valor_string = '0'

        # Codigo para dinamizar a criacao de instancias de entidade,
        # aplicando os valores dos atributos na instanciacao
        for k, v in kwargs.items():
            setattr(self, k, v)

        if kwargs.has_key('valor'):
            self.valor = kwargs['valor']

    def set_valor(self, novo_valor):
        if isinstance(novo_valor, basestring):
            if novo_valor:
                novo_valor = int(novo_valor)
            else:
                novo_valor = 0

        if isinstance(novo_valor, (int, long, Decimal)) and self._valida(novo_valor):
            self._valor_inteiro = novo_valor
            self._valor_string = unicode(self._valor_inteiro)

            if (len(self.tamanho) >= 3) and self.tamanho[2] and (len(self._valor_string) < self.tamanho[2]):
                self._valor_string = self._valor_string.rjust(self.tamanho[2], '0')

        else:
            self._valor_inteiro = 0
            self._valor_string = '0'

    def get_valor(self):
        return self._valor_inteiro

    valor = property(get_valor, set_valor)

    @property
    def formato_danfe(self):
        if not (self.obrigatorio or self._valor_inteiro):
            return ''

        return locale.format('%d', self._valor_inteiro, grouping=True)


class TagDecimal(TagCaracter):
    def __init__(self, *args, **kwargs):
        self._valor_decimal = Decimal('0.0')
        self._valor_string = '0.0'
        self.decimais = [None, None, None]
        super(TagDecimal, self).__init__(*args, **kwargs)

        self._valor_decimal = Decimal('0.0')
        self._valor_string = self._formata(self._valor_decimal)
        self.decimais = [None, None, None]

        # Codigo para dinamizar a criacao de instancias de entidade,
        # aplicando os valores dos atributos na instanciacao
        for k, v in kwargs.items():
            setattr(self, k, v)

    def _parte_inteira(self, valor=None):
        if valor is None:
            valor = self._valor_decimal

        valor = unicode(valor).strip()

        if '.' in valor:
            valor = valor.split('.')[0]

        return valor

    def _parte_decimal(self, valor=None):
        if valor is None:
            valor = self._valor_decimal

        valor = unicode(valor).strip()

        if '.' in valor:
            valor = valor.split('.')[1]
        else:
            valor = ''

        return valor

    def _formata(self, valor):
        texto = self._parte_inteira(valor)

        dec = self._parte_decimal(valor)
        if not dec:
            dec = '0'

        # Tamanho mínimo das casas decimais
        if (len(self.decimais) >= 3) and self.decimais[2] and (len(dec) < self.decimais[2]):
            dec = dec.ljust(self.decimais[2], '0')

        texto += '.' + dec
        return texto

    def _testa_decimais_minimo(self, decimal):
        if self.decimais[0] and (len(decimal) < self.decimais[0]):
            #return TamanhoInvalido(self.codigo, self.nome, decimal, dec_min=self.decimais[0])
            raise TamanhoInvalido(self.codigo, self.nome, decimal, dec_min=self.decimais[0])

    def _testa_decimais_maximo(self, decimal):
        if self.decimais[1] and (len(decimal) > self.decimais[1]):
            #return TamanhoInvalido(self.codigo, self.nome, decimal, dec_max=self.decimais[1])
            raise TamanhoInvalido(self.codigo, self.nome, decimal, dec_max=self.decimais[1])

    def _valida(self, valor):
        self.alertas = []

        if self._testa_obrigatorio(valor):
            self.alertas.append(self._testa_obrigatorio(valor))

        inteiro = self._parte_inteira(valor)
        decimal = self._parte_decimal(valor)

        if self._testa_tamanho_minimo(inteiro):
            self.alertas.append(self._testa_tamanho_minimo(inteiro))

        if self._testa_tamanho_maximo(inteiro):
            self.alertas.append(self._testa_tamanho_maximo(inteiro))

        #
        # Analisando as exp.reg. de validação das tags com decimais,
        # parece haver um número máximo de casas decimais, mas as tags
        # podem ser enviadas sem nenhuma casa decimal, então, não
        # há um mínimo de casas decimais
        #
        #if self._testa_decimais_minimo(decimal):
        #    self.alertas.append(self._testa_decimais_minimo(decimal))

        if self._testa_decimais_maximo(decimal):
            self.alertas.append(self._testa_decimais_maximo(decimal))

        return self.alertas == []

    def set_valor(self, novo_valor):
        if isinstance(novo_valor, basestring):
            if novo_valor:
                novo_valor = Decimal(novo_valor)
            else:
                novo_valor = Decimal('0.0')

        if isinstance(novo_valor, (int, long, Decimal)) and self._valida(novo_valor):
            self._valor_decimal = Decimal(novo_valor)
            self._valor_string = self._formata(self._valor_decimal)
        else:
            self._valor_decimal = Decimal('0.0')
            self._valor_string = self._formata(self._valor_decimal)

    def get_valor(self):
        return self._valor_decimal

    valor = property(get_valor, set_valor)

    @property
    def formato_danfe(self):
        if not (self.obrigatorio or self._valor_decimal):
            return ''

        # Tamanho mínimo das casas decimais
        if (len(self.decimais) >= 3) and self.decimais[2]:
            if len(self._parte_decimal()) <= self.decimais[2]:
                formato = '%.' + unicode(self.decimais[2]) + 'f'
            else:
                formato = '%.' + unicode(len(self._parte_decimal())) + 'f'
        else:
            formato = '%.2f'

        return locale.format(formato, self._valor_decimal, grouping=True)

    @property
    def formato_danfce(self):
        if not (self.obrigatorio or self._valor_decimal):
            return ''

        # Tamanho mínimo das casas decimais
        #if (len(self.decimais) >= 3) and self.decimais[2]:
            #if len(self._parte_decimal()) <= self.decimais[2]:
                #formato = '%.' + unicode(self.decimais[2]) + 'f'
            #else:
                #formato = '%.' + unicode(len(self._parte_decimal())) + 'f'
        #else:
        formato = '%.2f'

        return locale.format(formato, self._valor_decimal, grouping=True)


class XMLNFe(NohXML):
    def __init__(self, *args, **kwargs):
        super(XMLNFe, self).__init__(*args, **kwargs)
        self._xml = None
        self.alertas = []
        self.arquivo_esquema = None
        self.caminho_esquema = None

    def get_xml(self):
        self.alertas = []
        return ''

    def validar(self):
        arquivo_esquema = self.caminho_esquema + self.arquivo_esquema

        # Aqui é importante remover a declaração do encoding
        # para evitar erros de conversão unicode para ascii
        xml = tira_abertura(self.xml).encode('utf-8')

        esquema = etree.XMLSchema(etree.parse(arquivo_esquema))
        #esquema.assertValid(etree.fromstring(xml))
        esquema.validate(etree.fromstring(xml))

        return esquema.error_log

    def le_grupo(self, raiz_grupo, classe_grupo, sigla_ns='nfe'):
        tags = []

        grupos = self._le_nohs(raiz_grupo, sigla_ns=sigla_ns)

        if grupos is not None:
            tags = [classe_grupo() for g in grupos]
            for i in range(len(grupos)):
                tags[i].xml = grupos[i]

        return tags


def tirar_acentos(texto):
    if not texto:
        return texto

    texto = texto.replace('&', '&amp;')
    texto = texto.replace('<', '&lt;')
    texto = texto.replace('>', '&gt;')
    texto = texto.replace('"', '&quot;')
    texto = texto.replace("'", '&apos;')

    #
    # Trocar ENTER e TAB
    #
    texto = texto.replace('\t', ' ')
    texto = texto.replace('\n', '| ')

    # Remove espaços seguidos
    # Nem pergunte...
    while '  ' in texto:
        texto = texto.replace('  ', ' ')

    return texto

def por_acentos(texto):
    if not texto:
        return texto

    texto = texto.replace('&#39;', "'")
    texto = texto.replace('&apos;', "'")
    texto = texto.replace('&quot;', '"')
    texto = texto.replace('&gt;', '>')
    texto = texto.replace('&lt;', '<')
    texto = texto.replace('&amp;', '&')
    texto = texto.replace('&APOS;', "'")
    texto = texto.replace('&QUOT;', '"')
    texto = texto.replace('&GT;', '>')
    texto = texto.replace('&LT;', '<')
    texto = texto.replace('&AMP;', '&')

    return texto

def tira_abertura(texto):
    #aberturas = (
        #'<?xml version="1.0" encoding="utf-8"?>',
        #'<?xml version="1.0" encoding="utf-8" ?>',
        #'<?xml version="1.0" encoding="utf-8" standalone="no"?>',
        #'<?xml version="1.0" encoding="utf-8" standalone="no" ?>',
        #'<?xml version="1.0" encoding="utf-8" standalone="yes"?>',
        #'<?xml version="1.0" encoding="utf-8" standalone="yes" ?>',

        #'<?xml version="1.0" encoding="UTF-8"?>',
        #'<?xml version="1.0" encoding="UTF-8" ?>',
        #'<?xml version="1.0" encoding="UTF-8" standalone="no"?>',
        #'<?xml version="1.0" encoding="UTF-8" standalone="no" ?>',
        #'<?xml version="1.0" encoding="UTF-8" standalone="yes"?>',
        #'<?xml version="1.0" encoding="UTF-8" standalone="yes" ?>',

        #"<?xml version='1.0' encoding='utf-8'?>",
        #"<?xml version='1.0' encoding='utf-8' ?>",
        #"<?xml version='1.0' encoding='utf-8' standalone='no'?>",
        #"<?xml version='1.0' encoding='utf-8' standalone='no' ?>",
        #"<?xml version='1.0' encoding='utf-8' standalone='yes'?>",
        #"<?xml version='1.0' encoding='utf-8' standalone='yes' ?>",

        #"<?xml version='1.0' encoding='UTF-8'?>",
        #"<?xml version='1.0' encoding='UTF-8' ?>",
        #"<?xml version='1.0' encoding='UTF-8' standalone='no'?>",
        #"<?xml version='1.0' encoding='UTF-8' standalone='no' ?>",
        #"<?xml version='1.0' encoding='UTF-8' standalone='yes'?>",
        #"<?xml version='1.0' encoding='UTF-8' standalone='yes' ?>",
        #)

    #for a in aberturas:
        #texto = texto.replace(a,  '')

    if '?>' in texto:
        texto = texto.split('?>')[1:]
        texto = ''.join(texto)

    return texto

def _tipo_para_string(valor, tipo, obrigatorio, dec_min):
    if (not obrigatorio) and (not valor):
        return '', ''

    decimais = ''

    # Cuidado!!!
    # Aqui não dá pra usar a função strftime pois em alguns
    # casos a data retornada é 01/01/0001 00:00:00
    # e a função strftime só aceita data com anos a partir de 1900
    if (tipo in ('d', 'h', 'dh')) and isinstance(valor, (datetime, date, time,)):
        valor = formata_datahora(valor, tipo)
    elif (tipo == 'n') and isinstance(valor, (int, long, float, Decimal)):
        if isinstance(valor, (int, long, float)):
            valor = Decimal(unicode(valor))

        valor = unicode(valor).strip()

        if '.' in valor:
            decimais = valor.split('.')[1]

        if dec_min:
            decimais = decimais.ljust(dec_min, '0')

            if '.' in valor:
                valor = valor.split('.')[0]

            valor += '.' + decimais

    return valor, decimais

def _string_para_tipo(valor, tipo):
    if valor == None:
        return valor

    if tipo == 'd':
        valor = datetime.strptime(valor, b'%Y-%m-%d')
    elif tipo == 'h':
        valor = datetime.strptime(valor, b'%H:%M:%S')
    elif tipo == 'dh':
        valor = datetime.strptime(valor, b'%Y-%m-%dT%H:%M:%S')
    elif tipo == 'n':
        valor = Decimal(valor)

    return valor

def formata_datahora(valor, tipo):
    if (tipo == 'd') and isinstance(valor, (datetime, date,)):
        valor = '%04d-%02d-%02d' % (valor.year, valor.month, valor.day)
    elif (tipo == 'h') and isinstance(valor, (datetime, time,)):
        valor = '%02d:%02d:%02d' % (valor.hour, valor.minute, valor.second)
        valor = valor.strftime('%H:%M:%S')
    elif (tipo == 'dh') and isinstance(valor, datetime):
        valor = '%04d-%02d-%02dT%02d:%02d:%02d' % (valor.year, valor.month, valor.day, valor.hour, valor.minute, valor.second)

    return valor

def somente_ascii(funcao):
    '''
    Usado como decorator para a nota fiscal eletrônica de servicos
    '''
    def converter_para_ascii_puro(*args, **kwargs):
        return unicodedata.normalize(b'NFKD', funcao(*args, **kwargs)).encode('ascii', 'ignore')

    return converter_para_ascii_puro




# -*- coding: utf-8 -*-
#
# PySPED - Python libraries to deal with Brazil's SPED Project
#
# Copyright (C) 2010-2012
# Copyright (C) Aristides Caldeira <aristides.caldeira at tauga.com.br>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Library General Public License as
# published by the Free Software Foundation, either version 2.1 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# PySPED - Bibliotecas Python para o
#          SPED - Sistema Público de Escrituração Digital
#
# Copyright (C) 2010-2012
# Copyright (C) Aristides Caldeira <aristides.caldeira arroba tauga.com.br>
#
# Este programa é um software livre: você pode redistribuir e/ou modificar
# este programa sob os termos da licença GNU Library General Public License,
# publicada pela Free Software Foundation, em sua versão 2.1 ou, de acordo
# com sua opção, qualquer versão posterior.
#
# Este programa é distribuido na esperança de que venha a ser útil,
# porém SEM QUAISQUER GARANTIAS, nem mesmo a garantia implícita de
# COMERCIABILIDADE ou ADEQUAÇÃO A UMA FINALIDADE ESPECÍFICA. Veja a
# GNU Library General Public License para mais detalhes.
#
# Você deve ter recebido uma cópia da GNU Library General Public License
# juntamente com este programa. Caso esse não seja o caso, acesse:
# <http://www.gnu.org/licenses/>
#

from __future__ import division, print_function, unicode_literals

from pysped.xml_sped import NAMESPACE_NFE, TagCaracter, TagDecimal, XMLNFe
from pysped.nfe.leiaute import ESQUEMA_ATUAL_VERSAO_2 as ESQUEMA_ATUAL
from pysped.nfe.leiaute import consrecinfe_110
import os
from nfe_200 import NFe


DIRNAME = os.path.dirname(__file__)


class ConsReciNFe(consrecinfe_110.ConsReciNFe):
    def __init__(self):
        super(ConsReciNFe, self).__init__()
        self.versao  = TagDecimal(nome='consReciNFe', codigo='BP02', propriedade='versao', namespace=NAMESPACE_NFE, valor='2.00', raiz='/')
        self.caminho_esquema = os.path.join(DIRNAME, 'schema/', ESQUEMA_ATUAL + '/')
        self.arquivo_esquema = 'consReciNFe_v2.00.xsd'


class InfProt(consrecinfe_110.InfProt):
    def __init__(self):
        super(InfProt, self).__init__()


class ProtNFe(consrecinfe_110.ProtNFe):
    def __init__(self):
        super(ProtNFe, self).__init__()
        self.versao  = TagDecimal(nome='protNFe', codigo='PR02' , propriedade='versao', namespace=NAMESPACE_NFE, valor='2.00', raiz='/')


class RetConsReciNFe(consrecinfe_110.RetConsReciNFe):
    def __init__(self):
        super(RetConsReciNFe, self).__init__()
        self.versao   = TagDecimal(nome='retConsReciNFe', codigo='BR02' , propriedade='versao', namespace=NAMESPACE_NFE, valor='2.00', raiz='/')
        self.cMsg     = TagCaracter(nome='cMsg'         , codigo='BR06b', tamanho=[4,   4], raiz='//retConsReciNFe', obrigatorio=False)
        self.xMsg     = TagCaracter(nome='xMsg'         , codigo='BR06c', tamanho=[1, 200], raiz='//retConsReciNFe', obrigatorio=False)
        self.caminho_esquema = os.path.join(DIRNAME, 'schema/', ESQUEMA_ATUAL + '/')
        self.arquivo_esquema = 'retConsReciNFe_v2.00.xsd'


    def get_xml(self):
        xml = XMLNFe.get_xml(self)
        xml += self.versao.xml
        xml += self.tpAmb.xml
        xml += self.verAplic.xml
        xml += self.nRec.xml
        xml += self.cStat.xml
        xml += self.xMotivo.xml
        xml += self.cUF.xml
        xml += self.cMsg.xml
        xml += self.xMsg.xml

        for pn in self.protNFe:
            xml += pn.xml

        xml += '</retConsReciNFe>'
        return xml

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.versao.xml   = arquivo
            self.tpAmb.xml    = arquivo
            self.verAplic.xml = arquivo
            self.nRec.xml     = arquivo
            self.cStat.xml    = arquivo
            self.xMotivo.xml  = arquivo
            self.cUF.xml      = arquivo
            self.cMsg.xml     = arquivo
            self.xMsg.xml     = arquivo
            self.protNFe      = self.le_grupo('//retConsReciNFe/protNFe', ProtNFe)

            #
            # Monta o dicionário dos protocolos
            #
            for pn in self.protNFe:
                self.dic_protNFe[pn.infProt.chNFe.valor] = pn

    xml = property(get_xml, set_xml)


class ProcNFe(consrecinfe_110.ProcNFe):
    def __init__(self):
        super(ProcNFe, self).__init__()
        self.versao  = TagDecimal(nome='nfeProc', propriedade='versao', namespace=NAMESPACE_NFE, valor='2.00', raiz='/')
        self.NFe     = NFe()
        self.protNFe = ProtNFe()
        self.caminho_esquema = os.path.join(DIRNAME, 'schema/', ESQUEMA_ATUAL + '/')
        self.arquivo_esquema = 'procNFe_v2.00.xsd'



# -*- coding: utf-8 -*-
#
# PySPED - Python libraries to deal with Brazil's SPED Project
#
# Copyright (C) 2010-2012
# Copyright (C) Aristides Caldeira <aristides.caldeira at tauga.com.br>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Library General Public License as
# published by the Free Software Foundation, either version 2.1 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# PySPED - Bibliotecas Python para o
#          SPED - Sistema Público de Escrituração Digital
#
# Copyright (C) 2010-2012
# Copyright (C) Aristides Caldeira <aristides.caldeira arroba tauga.com.br>
#
# Este programa é um software livre: você pode redistribuir e/ou modificar
# este programa sob os termos da licença GNU Library General Public License,
# publicada pela Free Software Foundation, em sua versão 2.1 ou, de acordo
# com sua opção, qualquer versão posterior.
#
# Este programa é distribuido na esperança de que venha a ser útil,
# porém SEM QUAISQUER GARANTIAS, nem mesmo a garantia implícita de
# COMERCIABILIDADE ou ADEQUAÇÃO A UMA FINALIDADE ESPECÍFICA. Veja a
# GNU Library General Public License para mais detalhes.
#
# Você deve ter recebido uma cópia da GNU Library General Public License
# juntamente com este programa. Caso esse não seja o caso, acesse:
# <http://www.gnu.org/licenses/>
#

from __future__ import division, print_function, unicode_literals

import os
import re
from lxml import etree

from leiaute import *
from pysped.xml_sped import XMLNFe, tira_abertura
from pysped.xml_sped.certificado import Certificado


MAPA_ESQUEMAS = {}

for tipo in dir():
    if type(eval(tipo)) == type and issubclass(eval(tipo), XMLNFe) and eval(tipo + '().arquivo_esquema'):
            MAPA_ESQUEMAS[tipo] = os.path.join(eval(tipo + '().caminho_esquema'), eval(tipo + '().arquivo_esquema'))


LIMPA_ESPACO_ENTRE_TAGS = re.compile(r'(>)\s+(<)')


def valida_codificacao(arq_xml):
    try:
        xml = open(arq_xml).read()
        xml = xml.decode('utf-8')

        #
        # Remove caracteres desnecessários
        #
        xml = xml.replace('\r', '')
        xml = xml.replace('\n', '')

        #
        # Remove espaços entre as tags
        #
        xml = LIMPA_ESPACO_ENTRE_TAGS.sub(r'\1\2', xml)

        #
        # Tira tag de abertura com a codificação
        #
        xml = tira_abertura(xml)
    except:
        return ''

    return xml


def correcoes_xml(xml):
    CORRECOES = [
        #
        # Corrige falta de versão no CT-e
        #
        ('<cteProc xmlns="http://www.portalfiscal.inf.br/cte" versao="">', '<cteProc xmlns="http://www.portalfiscal.inf.br/cte" versao="1.04">'),
        ('<cteProc versao="1.04" xmlns="http://www.portalfiscal.inf.br/cte" xmlns:ds="http://www.w3.org/2000/09/xmldsig#" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://www.portalfiscal.inf.br/cte procCTe_v1.04.xsd">', '<cteProc xmlns="http://www.portalfiscal.inf.br/cte" versao="1.04">'),

        #
        # Corrige schema da assinatura no lugar errado
        #
        ('<nfeProc xmlns="http://www.portalfiscal.inf.br/nfe" xmlns:ds="http://www.w3.org/2000/09/xmldsig#" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://www.portalfiscal.inf.br/nfe procNFe_v2.00.xsd" versao="2.00">', '<nfeProc versao="2.00" xmlns="http://www.portalfiscal.inf.br/nfe">'),

        ('<nfeProc versao="2.00" xmlns="http://www.portalfiscal.inf.br/nfe" xmlns:ds="http://www.w3.org/2000/09/xmldsig#" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance&quot;">', '<nfeProc versao="2.00" xmlns="http://www.portalfiscal.inf.br/nfe">'),

        ('<nfeProc versao="2.00" xmlns:ns2="http://www.w3.org/2000/09/xmldsig#" xmlns="http://www.portalfiscal.inf.br/nfe">', '<nfeProc versao="2.00" xmlns="http://www.portalfiscal.inf.br/nfe">'),

        ('<nfeProc xmlns="http://www.portalfiscal.inf.br/nfe" xmlns:ds="http://www.w3.org/2000/09/xmldsig#" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" versao="2.00">', '<nfeProc versao="2.00" xmlns="http://www.portalfiscal.inf.br/nfe">'),

        ('<nfeProc versao="2.00" xmlns="http://www.portalfiscal.inf.br/nfe" xmlns:ds="http://www.w3.org/2000/09/xmldsig#" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://www.portalfiscal.inf.br/nfe procNFe_v2.00.xsd">', '<nfeProc versao="2.00" xmlns="http://www.portalfiscal.inf.br/nfe">'),

        ('<nfeProc xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" versao="2.00" xmlns="http://www.portalfiscal.inf.br/nfe">', '<nfeProc versao="2.00" xmlns="http://www.portalfiscal.inf.br/nfe">'),

        ('<nfeProc xmlns="http://www.portalfiscal.inf.br/nfe" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" versao="2.00">', '<nfeProc versao="2.00" xmlns="http://www.portalfiscal.inf.br/nfe">'),

        ('<nfeProc versao="2.00" xmlns="http://www.portalfiscal.inf.br/nfe" xmlns:ds="http://www.w3.org/2000/09/xmldsig#" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">', '<nfeProc versao="2.00" xmlns="http://www.portalfiscal.inf.br/nfe">'),
    ]

    for errado, certo in CORRECOES:
        if errado in xml:
            xml = xml.replace(errado, certo)

    return xml


def valida_esquema(xml, tipo):
    arquivo_esquema = MAPA_ESQUEMAS[tipo]
    esquema = etree.XMLSchema(etree.parse(arquivo_esquema))

    #try:
    esquema.validate(etree.fromstring(xml.encode('utf-8')))
    erros = esquema.error_log
    #except:
        #return [False]

    return erros


def identifica_xml(xml, cria_instancia=False):
    tipos = []

    for tipo in MAPA_ESQUEMAS:
        erros = valida_esquema(xml, tipo)

        if len(erros) == 0:
            #
            # Tenta identificar o tipo exato do evento
            #
            if issubclass(eval(tipo), EnvEvento_100):
                if not tipo == 'EnvEvento_100':
                    #if valida_esquema.evento[0].infEvento.tpEvento.xml in arq:
                    tipos += [tipo]

            elif issubclass(eval(tipo), RetEnvEvento_100):
                if not tipo == 'RetEnvEvento_100':
                    #if valida_esquema.retEvento[0].infEvento.tpEvento.xml in arq:
                    tipos += [tipo]

            elif issubclass(eval(tipo), Evento_100):
                if not tipo == 'Evento_100':
                    #if valida_esquema.infEvento.tpEvento.xml in arq:
                    tipos += [tipo]

            elif issubclass(eval(tipo), RetEvento_100):
                if not tipo == 'RetEvento_100':
                    #if valida_esquema.infEvento.tpEvento.xml in arq:
                    tipos += [tipo]

            elif issubclass(eval(tipo), ProcEvento_100):
                if not tipo == 'ProcEvento_100':
                    #if valida_esquema.evento.infEvento.tpEvento.xml in arq:
                    tipos += [tipo]

            else:
                tipos += [tipo]

    if cria_instancia:
        instancia = eval(tipos[0] + '()')
        instancia.xml = xml
        return instancia, tipos[0]

    return tipos


def valida_assinatura(xml, arq_xml, tipo):
    if tipo not in ('NFe_110', 'CancNFe_107', 'InutNFe_107', 'ProcNFe_110', 'EnviNFe_110',
                    'NFe_200', 'CancNFe_200', 'InutNFe_200', 'ProcNFe_200', 'EnviNFe_200',
                    'NFe_310', 'InutNFe_310', 'ProcNFe_310', 'EnviNFe_310',
                    'CTe_104', 'CancCTe_104', 'InutCTe_104', 'ProcCTe_104', 'EnviCTe_104',
                    'Evento_100', 'EnvEvento_100', 'ProcEvento_100',
                    'EventoCCe_100', 'EnvEventoCCe_100', 'ProcEventoCCe_100',
                    'EventoCancNFe_100', 'EnvEventoCancNFe_100', 'ProcEventoCancNFe_100',
                    'EventoConfRecebimento_100', 'EnvEventoConfRecebimento_100', 'ProcEventoConfRecebimento_100'):
        return True

    assinatura_valida = False
    try:
        c = Certificado()
        assinatura_valida = c.verifica_assinatura_xml(xml)
    except:
        pass

    if not assinatura_valida:
        try:
            assinatura_valida = c.verifica_assinatura_arquivo(arq_xml)
        except:
            pass

    return assinatura_valida
